import os
import glob
import argparse
from Bio import SeqIO
from Bio import AlignIO


AssessMarkerPA_usage = '''
=========================== AssessMarkerPA example commands ===========================

BioSAK AssessMarkerPA -ta trimmed_aln -tax aln -aa faa_files -aax faa -g gnm_group.txt -c 25-50-75-100 -o s10_assess_marker_PA

Note
1. Extra genomes in gnm_metadata.txt won't affect assessment results.
2. Genomes can not be found in gnm_metadata.txt will trigger an error.
3. Alignments in {trimmed_aln_dir} need to be trimmed before assessment
4. Sequences in MSAs need to be named by genome id.

=======================================================================================
'''


def sep_path_basename_ext(file_in):
    file_path, file_name = os.path.split(file_in)
    if file_path == '':
        file_path = '.'
    file_basename, file_extension = os.path.splitext(file_name)
    return file_path, file_basename, file_extension


def catfasta2phy(msa_dir, msa_ext, concatenated_msa_phy, partition_file):

    concatenated_msa_fasta = '%s.fasta' % concatenated_msa_phy
    msa_file_re            = '%s/*.%s'  % (msa_dir, msa_ext)
    msa_file_list          = [os.path.basename(file_name) for file_name in glob.glob(msa_file_re)]
    msa_file_list_sorted   = sorted(msa_file_list)

    complete_gnm_set = set()
    for each_msa_file in msa_file_list:
        pwd_msa = '%s/%s' % (msa_dir, each_msa_file)
        for each_seq in SeqIO.parse(pwd_msa, 'fasta'):
            complete_gnm_set.add(each_seq.id)

    complete_gnm_list_sorted = sorted([i for i in complete_gnm_set])

    # initialize concatenated msa dict
    gnm_to_seq_dict = {i: '' for i in complete_gnm_list_sorted}
    msa_len_dict = dict()
    for each_msa_file in msa_file_list_sorted:
        gene_id = each_msa_file.split('.' + msa_ext)[0]

        # read in msa
        current_msa_len = 0
        current_msa_len_set = set()
        pwd_current_msa = '%s/%s' % (msa_dir, each_msa_file)
        current_msa_seq_dict = dict()
        for each_seq in SeqIO.parse(pwd_current_msa, 'fasta'):
            complete_gnm_set.add(each_seq.id)
            current_msa_seq_dict[each_seq.id] = str(each_seq.seq)
            current_msa_len_set.add(len(each_seq.seq))
            current_msa_len = len(each_seq.seq)

        if len(current_msa_len_set) != 1:
            print('Sequences with different length were found in %s, program exited!' % each_msa_file)
            exit()

        msa_len_dict[gene_id] = current_msa_len

        # add sequence to concatenated msa dict
        for each_gnm in complete_gnm_list_sorted:
            msa_seq = current_msa_seq_dict.get(each_gnm, current_msa_len*'-')
            gnm_to_seq_dict[each_gnm] += msa_seq

    # write out concatenated msa
    concatenated_msa_handle = open(concatenated_msa_fasta, 'w')
    for each_gnm in complete_gnm_list_sorted:
        concatenated_msa_handle.write('>%s\n' % each_gnm)
        concatenated_msa_handle.write('%s\n' % gnm_to_seq_dict[each_gnm])
    concatenated_msa_handle.close()

    # write out partition file
    end_pos = 0
    partition_file_handle = open(partition_file, 'w')
    for each_m in msa_file_list_sorted:
        gene_id = each_m.split('.' + msa_ext)[0]
        current_m_len = msa_len_dict[gene_id]
        partition_file_handle.write('%s = %s-%s\n' % (each_m, (end_pos + 1), (end_pos + current_m_len)))
        end_pos += current_m_len
    partition_file_handle.close()

    # convert msa in fasta to phy
    AlignIO.convert(concatenated_msa_fasta, 'fasta', concatenated_msa_phy, 'phylip-relaxed')


def AssessMarkerPA(args):

    trimmed_aln_dir   = args['ta']
    trimmed_aln_ext   = args['tax']
    faa_file_dir      = args['aa']
    faa_file_ext      = args['aax']
    gnm_group_txt     = args['g']
    cutoff_str        = args['c']
    op_dir            = args['o']
    force_overwriting = args['f']
    #catfasta2phyml_pl = args['pl']
    js_cpu_num        = args['jst']

    # get gnm id list
    faa_file_re   = '%s/*.%s' % (faa_file_dir, faa_file_ext)
    faa_file_list = [os.path.basename(file_name) for file_name in glob.glob(faa_file_re)]
    gnm_set = set()
    for each_faa_file in faa_file_list:
        faa_path, faa_basename, faa_ext = sep_path_basename_ext(each_faa_file)
        gnm_set.add(faa_basename)

    # read in genome metadata
    group_to_gnm_dict       = dict()
    group_to_gnm_num_dict   = dict()
    gnm_to_group_dict       = dict()
    for each_gnm in open(gnm_group_txt):
        each_gnm_split = each_gnm.strip().split('\t')
        gnm_id = each_gnm_split[0]
        domain_name = each_gnm_split[1]

        if gnm_id in gnm_set:
            gnm_to_group_dict[gnm_id] = domain_name

            if domain_name not in group_to_gnm_num_dict:
                group_to_gnm_num_dict[domain_name] = 1
            else:
                group_to_gnm_num_dict[domain_name] += 1

            if domain_name not in group_to_gnm_dict:
                group_to_gnm_dict[domain_name] = {gnm_id}
            else:
                group_to_gnm_dict[domain_name].add(gnm_id)

    group_id_list_sorted = sorted(list(group_to_gnm_dict.keys()))

    # exit program if group information is missing
    gnms_without_group_info = set()
    for gnm in gnm_set:
        if gnm not in gnm_to_group_dict:
            gnms_without_group_info.add(gnm)

    if len(gnms_without_group_info) > 0:
        print('Group information for the following genomes are missing from %s, program exited!' % gnm_group_txt)
        print(','.join(gnms_without_group_info))
        print('Group information for the above genomes are missing from %s, program exited!' % gnm_group_txt)
        exit()

    # create folder
    if force_overwriting is True:
        if os.path.isdir(op_dir) is True:
            os.system('rm -r %s' % op_dir)
    else:
        if os.path.isdir(op_dir) is True:
            print('Output folder already exist, program exited!')
            exit()
    os.system('mkdir %s' % op_dir)

    # read in provided cutoffs
    present_pct_cutoff_list = [int(i) for i in cutoff_str.split('-')]
    assess_summary_1_txt    = '%s/assessment_PA.txt'                % op_dir
    assess_summary_2_txt    = '%s/assessment_PA_summary.txt'        % op_dir
    itol_binary_txt         = '%s/assessment_PA_iTOL_binary.txt'    % op_dir

    trimmed_aln_file_re = '%s/*.%s' % (trimmed_aln_dir, trimmed_aln_ext)
    trimmed_aln_file_list = [os.path.basename(file_name) for file_name in glob.glob(trimmed_aln_file_re)]

    assess_summary_1_txt_handle = open(assess_summary_1_txt, 'w')
    assess_summary_1_txt_handle.write('Marker\t%s\n' % '\t'.join([str(i) for i in group_id_list_sorted]))
    assess_summary_2_txt_handle = open(assess_summary_2_txt, 'w')
    assess_summary_2_txt_handle.write('Marker\t%s\n' % '\t'.join([str(i) for i in present_pct_cutoff_list]))
    cutoff_to_qualified_marker_dict = dict()
    gnm_to_identified_marker_dict = dict()
    marker_id_list = []
    for each_aln in trimmed_aln_file_list:

        marker_id = each_aln.split(('.%s' % trimmed_aln_ext))[0]
        marker_id_list.append(marker_id)
        pwd_aln = '%s/%s' % (trimmed_aln_dir, each_aln)

        current_marker_num_by_group_dict = dict()
        for each_seq in SeqIO.parse(pwd_aln, 'fasta'):
            gnm_id = each_seq.id

            # get genome to marker dist
            if gnm_id not in gnm_to_identified_marker_dict:
                gnm_to_identified_marker_dict[gnm_id] = {marker_id}
            else:
                gnm_to_identified_marker_dict[gnm_id].add(marker_id)

            if gnm_id in gnm_to_group_dict:
                gnm_group = gnm_to_group_dict[gnm_id]
                if gnm_group not in current_marker_num_by_group_dict:
                    current_marker_num_by_group_dict[gnm_group] = 1
                else:
                    current_marker_num_by_group_dict[gnm_group] += 1
            else:
                print('Not all genomes used to generate the MSA being found in -aa, program exited!')
                exit()

        # write out assess_summary_1_txt
        pct_list = []
        for each_grp in group_id_list_sorted:
            grp_pct = current_marker_num_by_group_dict.get(each_grp, 0)*100/group_to_gnm_num_dict[each_grp]
            grp_pct = float("{0:.2f}".format(grp_pct))
            pct_list.append(grp_pct)
        assess_summary_1_txt_handle.write('%s\t%s\n' % (marker_id, '\t'.join([str(i) for i in pct_list])))

        # write out assess_summary_2_txt
        assess_list = []
        for each_cutoff in present_pct_cutoff_list:

            good_marker = True
            for each_pct in pct_list:
                if each_pct < each_cutoff:
                    good_marker = False

            if each_cutoff not in cutoff_to_qualified_marker_dict:
                cutoff_to_qualified_marker_dict[each_cutoff] = {marker_id}

            if good_marker is True:
                assess_list.append('1')
                cutoff_to_qualified_marker_dict[each_cutoff].add(marker_id)
            else:
                assess_list.append('0')
        assess_summary_2_txt_handle.write('%s\t%s\n' % (marker_id, '\t'.join(assess_list)))

    # write out total in assess_summary_2_txt
    total_stats_list = [str(len(cutoff_to_qualified_marker_dict[each_c])) for each_c in present_pct_cutoff_list]
    assess_summary_2_txt_handle.write('Total\t%s\n' % ('\t'.join(total_stats_list)))
    assess_summary_1_txt_handle.close()
    assess_summary_2_txt_handle.close()

    # copy alignments of qualified marker to corresponding folders
    for each_cutoff in cutoff_to_qualified_marker_dict:
        qualified_marker_set = cutoff_to_qualified_marker_dict[each_cutoff]

        qualified_marker_phy           = 'qualified_marker_PA_%s_concatenated.phy'               % each_cutoff
        pwd_qualified_marker_dir       = '%s/qualified_marker_PA_%s'                             % (op_dir, each_cutoff)
        pwd_qualified_marker_id_txt    = '%s/qualified_marker_PA_%s_id.txt'                      % (op_dir, each_cutoff)
        pwd_qualified_marker_phy       = '%s/qualified_marker_PA_%s_concatenated.phy'            % (op_dir, each_cutoff)
        pwd_qualified_marker_partition = '%s/qualified_marker_PA_%s_concatenated_partition.txt'  % (op_dir, each_cutoff)
        pwd_js_iqtree                  = '%s/js_iqtree_PA_%s.sh'                                 % (op_dir, each_cutoff)

        os.system('mkdir %s' % pwd_qualified_marker_dir)
        for each_marker in qualified_marker_set:
            pwd_marker_aln = '%s/%s.%s' % (trimmed_aln_dir, each_marker, trimmed_aln_ext)
            cp_cmd = 'cp %s %s/' % (pwd_marker_aln, pwd_qualified_marker_dir)
            os.system(cp_cmd)

        # write out id
        with open(pwd_qualified_marker_id_txt, 'w') as pwd_qualified_marker_id_txt_handle:
            pwd_qualified_marker_id_txt_handle.write('%s\n' % '\n'.join(qualified_marker_set))

        # concatenate qualified alignments
        #catfasta2phyml_cmd = 'perl %s --sequential --concatenate %s/*.aln > %s 2> %s' % (catfasta2phyml_pl, pwd_qualified_marker_dir, pwd_qualified_marker_phy, pwd_qualified_marker_partition)
        #print('running: ' + catfasta2phyml_cmd)
        #os.system(catfasta2phyml_cmd)
        catfasta2phy(pwd_qualified_marker_dir, 'aln', pwd_qualified_marker_phy, pwd_qualified_marker_partition)

        # write out iqtree js
        guide_tree_dir          = 'qualified_marker_PA_%s_guide_tree'       % each_cutoff
        iqtree_C60_PMSF_dir     = 'qualified_marker_PA_%s_iqtree_C60_PMSF'  % each_cutoff
        pwd_guide_tree_dir      = '%s/%s'                                   % (op_dir, guide_tree_dir)
        pwd_iqtree_C60_PMSF_dir = '%s/%s'                                   % (op_dir, iqtree_C60_PMSF_dir)

        with open(pwd_js_iqtree, 'w') as pwd_js_iqtree_handle:
            pwd_js_iqtree_handle.write('#!/bin/bash\n#SBATCH --ntasks 1\n#SBATCH --cpus-per-task %s\n' % js_cpu_num)
            pwd_js_iqtree_handle.write('mkdir %s/%s\n' % (os.getcwd(), pwd_guide_tree_dir))
            pwd_js_iqtree_handle.write('cd %s/%s\n'    % (os.getcwd(), pwd_guide_tree_dir))
            pwd_js_iqtree_handle.write('iqtree -s ../%s --prefix guide_tree --seqtype AA -m LG -T %s -B 1000 --alrt 1000\n' % (qualified_marker_phy, js_cpu_num))
            pwd_js_iqtree_handle.write('mkdir %s/%s\n' % (os.getcwd(), pwd_iqtree_C60_PMSF_dir))
            pwd_js_iqtree_handle.write('cd %s/%s\n'    % (os.getcwd(), pwd_iqtree_C60_PMSF_dir))
            pwd_js_iqtree_handle.write('iqtree -s ../%s --prefix concatenated --seqtype AA -m LG+G+F+C60 -T %s -B 1000 --alrt 1000 -ft ../%s/guide_tree.treefile\n' % (qualified_marker_phy, js_cpu_num, guide_tree_dir))

    # write out iTOL file
    itol_binary_txt_handle = open(itol_binary_txt, 'w')
    itol_binary_txt_handle.write('DATASET_BINARY\n\nSEPARATOR TAB\nDATASET_LABEL\tlabel1\nCOLOR\t#85C1E9\n')
    itol_binary_txt_handle.write('SHOW_LABELS\t1\nLABEL_ROTATION\t45\nLABEL_SHIFT\t5\n')
    itol_binary_txt_handle.write('FIELD_LABELS\t%s\n' % '\t'.join(sorted(marker_id_list)))
    itol_binary_txt_handle.write('FIELD_SHAPES\t%s\n' % '\t'.join(['1']*len(marker_id_list)))
    itol_binary_txt_handle.write('\nDATA\n')
    for each_g in gnm_to_identified_marker_dict:
        g_identified_marker_set = gnm_to_identified_marker_dict[each_g]

        pa_list = []
        for each_m in sorted(marker_id_list):
            if each_m in g_identified_marker_set:
                pa_list.append('1')
            else:
                pa_list.append('-1')
        itol_binary_txt_handle.write('%s\t%s\n' % (each_g, '\t'.join(pa_list)))
    itol_binary_txt_handle.close()

    print('Assessment results exported to:\n%s\n%s' % (assess_summary_1_txt, assess_summary_2_txt))
    print('Done!')


if __name__ == '__main__':

    parser = argparse.ArgumentParser()
    parser.add_argument('-ta',         required=True,                               help='trimmed alignments')
    parser.add_argument('-tax',        required=True,                               help='extension of trimmed alignments')
    parser.add_argument('-aa',         required=True,                               help='faa file dir')
    parser.add_argument('-aax',        required=True,                               help='faa file ext')
    parser.add_argument('-g',          required=True,                               help='genome group')
    parser.add_argument('-c',          required=False, default='50-75-100',         help='cutoffs, default: 50-75-100')
    parser.add_argument('-o',          required=True,                               help='output dir')
    parser.add_argument('-f',          required=False, action="store_true",         help='force overwrite existing output folder')
    #parser.add_argument('-pl',         required=True,                               help='path to catfasta2phyml.pl')
    parser.add_argument('-jst',        required=False, default='3',                 help='threads to request in job script')
    parser.add_argument('-qsub',       required=False, action="store_true",         help='submit job script')
    args = vars(parser.parse_args())
    AssessMarkerPA(args)



import os
import argparse


pRTC_usage = '''
==================================== pRTC example commands ====================================

# requires: ruby (bio-nwk, colorize, parallel, csv)

TreeSAK pRTC -i out -m mcmc.txt -r rtc_dir -o after_pRTC_mcmc.txt

python3 # format of RTC file (tab separated)
symbiont_1,symbiont_2    host_1,host_2:0.9876
recipient1,recipient2	 donor1,donor2:0.6789

# The above two lines can be interpreted as follows:
The probability of the last common ancestor (LCA) of symbiont_1 and symbiont_2 being 
younger than that of host_1 and host_2 is 0.9876. Similarly, for a gene transfer event, 
the LCA of the two recipients is younger than that of the two donors, the value 0.6789 
is the PROBABILITY of the occurrence of the transfer event. Please do NOT use this module 
if you are unsure about the interpretation of the value you provided.

# Note
This is a Python wrapper to perform the probabilistic RTC dating proposed by Dr. Sishuo Wang.
If you used it in your dating analysis, please cite:
https://doi.org/10.1101/2023.06.18.545440 or https://github.com/evolbeginner/rrtc.

===============================================================================================
'''


def pRTC(args):

    in_file     = args['i']
    mcmc_txt    = args['m']
    rrtc_dir    = args['r']
    ruby_exe    = args['ruby']
    op_txt      = args['o']

    current_file_path = '/'.join(os.path.realpath(__file__).split('/')[:-1])
    do_rrtc_rb        = '%s/do_rrtc.rb'                             % current_file_path
    do_rrtc_cmd       = '%s %s --mcmctxt %s -i %s --rrtc %s > %s'   % (ruby_exe, do_rrtc_rb, mcmc_txt, in_file, rrtc_dir, op_txt)
    print('Running: ' + do_rrtc_cmd)
    os.system(do_rrtc_cmd)


if __name__ == '__main__':

    pRTC_parser = argparse.ArgumentParser(usage=pRTC_usage)
    pRTC_parser.add_argument('-i',     required=True,                   help='the file "out" generated by MCMCTree')
    pRTC_parser.add_argument('-m',     required=True,                   help='the file "mcmc.txt" generated by MCMCTree')
    pRTC_parser.add_argument('-r',     required=True,                   help='the folder that contains all RTCs')
    pRTC_parser.add_argument('-o',     required=True,                   help='output txt file')
    pRTC_parser.add_argument('-ruby',  required=False, default='ruby',  help='path to ruby executable file, default: ruby')
    args = vars(pRTC_parser.parse_args())
    pRTC(args)



from __future__ import print_function
import os
import glob
import argparse
from Bio import SeqIO


SplitScore1_usage = '''
======================== SplitScore1 example commands ========================

TreeSAK SplitScore1 -i marker_seq -x fa -o SplitScore1_op_dir -jst 9 -f

# As described in the Undinarchaeota paper (Nina Dombrowski 2020, NC)

==============================================================================
'''


def sep_path_basename_ext(file_in):
    f_path, file_name = os.path.split(file_in)
    if f_path == '':
        f_path = '.'
    f_base, f_ext = os.path.splitext(file_name)
    return f_path, f_base, f_ext


def SplitScore1(args):

    oma_op_fasta        = args['i']
    fasta_file_ext      = args['x']
    interested_gnm_txt  = args['u']
    iqtree_model        = args['m']
    cov_cutoff          = args['c']
    force_overwrite     = args['f']
    num_of_js_threads   = args['jst']
    op_dir              = args['o']
    seq_named_by_gnm    = args['seq_named_by_gnm']
    bmge_trim_model             = 'BLOSUM30'
    bmge_entropy_score_cutoff   = '0.55'

    ################################################################################

    interested_gnm_set = set()
    if interested_gnm_txt is not None:
        if os.path.isfile(interested_gnm_txt):
            for each_gnm in open(interested_gnm_txt):
                interested_gnm_set.add(each_gnm.strip())
        else:
            print('%s not found, program exited' % interested_gnm_txt)
            exit()

    ################################################################################

    # specify path to BMGE.jar
    current_file_path = '/'.join(os.path.realpath(__file__).split('/')[:-1])
    pwd_bmge_jar      = '%s/BMGE.jar' % current_file_path

    fa_file_re   = '%s/*.%s' % (oma_op_fasta, fasta_file_ext)
    fa_file_list = glob.glob(fa_file_re)
    if len(fa_file_list) == 0:
        print('No file found in %s, program exited!' % oma_op_fasta)
        exit()

    og_to_gene_dict = dict()
    for each_fa in fa_file_list:
        _, f_base, _ = sep_path_basename_ext(each_fa)
        seq_id_set = set()
        for each_seq in SeqIO.parse(each_fa, 'fasta'):
            seq_id_set.add(each_seq.id)
        og_to_gene_dict[f_base] = seq_id_set

    ################################################################################

    gnm_to_process = set()
    for each_og in og_to_gene_dict:
        gene_set = og_to_gene_dict[each_og]
        gnm_set = set()
        for each_gene in gene_set:
            gnm_id = '_'.join(each_gene.split('_')[:-1])
            if seq_named_by_gnm is True:
                gnm_id = each_gene
            gnm_set.add(gnm_id)
            if interested_gnm_txt is None:
                gnm_to_process.add(gnm_id)
            else:
                if gnm_id in interested_gnm_set:
                    gnm_to_process.add(gnm_id)

        if len(gene_set) != len(gnm_set):
            print('Program exited!')
            exit()

    ################################################################################

    # define file name
    qualified_og_dir        = '%s/qualified_OGs'                % op_dir
    cmds_in_one_line_txt    = '%s/cmds_mafft_bmge_iqtree.txt'   % op_dir
    ignored_marker_txt      = '%s/ignored_markers.txt'          % op_dir

    # create output folder
    if os.path.isdir(op_dir) is True:
        if force_overwrite is True:
            os.system('rm -r %s' % op_dir)
        else:
            print('%s exist, program exited!' % op_dir)
            exit()
    os.mkdir(op_dir)
    os.mkdir(qualified_og_dir)

    ################################################################################

    cmds_in_one_line_txt_handle = open(cmds_in_one_line_txt, 'w')
    ignored_og_dict = dict()
    for each_og in sorted(list(og_to_gene_dict.keys())):
        seq_file_in          = '%s/%s.%s'       % (oma_op_fasta, each_og, fasta_file_ext)
        file_out_seq         = '%s/%s.%s'       % (qualified_og_dir, each_og, fasta_file_ext)
        file_out_aln         = '%s.aln'         % each_og
        file_out_aln_trimmed = '%s_trimmed.aln' % each_og

        seq_file_out_handle = open(file_out_seq, 'w')
        current_gnm_set = set()
        for each_seq in SeqIO.parse(seq_file_in, 'fasta'):
            seq_id = each_seq.id
            gnm_id = '_'.join(seq_id.split('_')[:-1])
            if seq_named_by_gnm is True:
                gnm_id = seq_id
            if gnm_id in gnm_to_process:
                current_gnm_set.add(gnm_id)
                seq_file_out_handle.write('>%s\n' % each_seq.id)
                seq_file_out_handle.write('%s\n'  % each_seq.seq)
        seq_file_out_handle.close()

        cov_value = len(current_gnm_set)*100/len(gnm_to_process)
        cov_value = float("{0:.2f}".format(cov_value))

        if cov_value < cov_cutoff:
            report_str = 'Ignored %s, contains proteins from %s (%s%s) genomes, < %s%s.' % (each_og, len(current_gnm_set), cov_value, '%', cov_cutoff, '%')
            ignored_og_dict[each_og] = report_str
            os.system('rm %s' % file_out_seq)
        else:
            # align, trim and iqtree
            mafft_cmd  = 'mafft-einsi --thread %s --quiet %s.%s > %s'                                % (num_of_js_threads, each_og, fasta_file_ext, file_out_aln)
            bmge_cmd   = 'java -jar %s -i %s -m %s -t AA -h %s -of %s'                               % (pwd_bmge_jar, file_out_aln, bmge_trim_model, bmge_entropy_score_cutoff, file_out_aln_trimmed)
            iqtree_cmd = 'iqtree2 -s %s --seqtype AA -m %s -B 1000 --wbtl --bnni --prefix %s -T %s --quiet' % (file_out_aln_trimmed, iqtree_model, each_og, num_of_js_threads)
            # Undinarchaeota illuminate DPANN phylogeny and the impact of gene transfer on archaeal evolution, settings: -m LG+G -bb 1000 -wbtl -bnni
            cmds_in_one_line_txt_handle.write('%s; %s; %s\n' % (mafft_cmd, bmge_cmd, iqtree_cmd))
    cmds_in_one_line_txt_handle.close()

    # report ignored markers
    if len(ignored_og_dict) > 0:
        print('The following %s markers were ignored due to low genome coverage, see details in %s:' % (len(ignored_og_dict), ignored_marker_txt))
        print('\n'.join(sorted(list(ignored_og_dict.keys()))))
        ignored_marker_txt_handle = open(ignored_marker_txt, 'w')
        for each_ignored_marker in sorted(list(ignored_og_dict.keys())):
            ignored_marker_txt_handle.write(ignored_og_dict[each_ignored_marker] + '\n')
        ignored_marker_txt_handle.close()

    # report
    print('You will need to execute the commands exported to the following file before moving to SplitScore2')
    print(cmds_in_one_line_txt)
    print('Done!')


if __name__ == '__main__':

    SplitScore1_parser = argparse.ArgumentParser()
    SplitScore1_parser.add_argument('-i',                   required=True,                          help='orthologous gene sequence')
    SplitScore1_parser.add_argument('-x',                   required=True,                          help='fasta file extension')
    SplitScore1_parser.add_argument('-o',                   required=True,                          help='output directory')
    SplitScore1_parser.add_argument('-u',                   required=False, default=None,           help='interested genomes, no file extension')
    SplitScore1_parser.add_argument('-m',                   required=False, default='LG+G',         help='iqtree_model, default: LG+G')
    SplitScore1_parser.add_argument('-c',                   required=False, type=int, default=75,   help='coverage cutoff, default: 75')
    SplitScore1_parser.add_argument('-f',                   required=False, action="store_true",    help='force overwrite')
    SplitScore1_parser.add_argument('-seq_named_by_gnm',    required=False, action="store_true",    help='named_by_gnm, specify if sequence named by gnm')
    SplitScore1_parser.add_argument('-jst',                 required=False, type=int, default=1,    help='num of threads for iqtree2, default: 1')
    args = vars(SplitScore1_parser.parse_args())
    SplitScore1(args)



import argparse
from ete3 import Tree


replace_clade_usage = '''
===================== replace_clade example commands =====================

TreeSAK replace_clade -m main.tree -s sub.tree -l leaves.txt -o out.tree

==========================================================================
'''


def replace_clade(args):

    main_tree_file  = args['m']
    sub_tree_file   = args['s']
    leaf_txt        = args['l']
    tree_out        = args['o']
    tree_out_fmt    = args['of']

    # read in subtree
    sub_tre = Tree(sub_tree_file, quoted_node_names=True, format=1)

    # read in leaves
    leaf_list = []
    for each_leaf in open(leaf_txt):
        leaf_list.append(each_leaf.strip())

    # read in main tree
    main_tre = Tree(main_tree_file, quoted_node_names=True, format=1)

    # remove clades
    lca = main_tre.get_common_ancestor(leaf_list)

    lca_p = lca.up
    lca_p.remove_child(lca)
    lca_p.add_child(sub_tre)

    # write out updated tree
    main_tre.write(outfile=tree_out, format=tree_out_fmt)




if __name__ == '__main__':

    replace_clade_parser = argparse.ArgumentParser()
    replace_clade_parser.add_argument('-m',   required=True,                        help='main tree file')
    replace_clade_parser.add_argument('-s',   required=True,                        help='subtree file')
    replace_clade_parser.add_argument('-l',   required=True,                        help='leaves on main tree to be replaced')
    replace_clade_parser.add_argument('-o',   required=True,                        help='output tree')
    replace_clade_parser.add_argument('-of',  required=False, default=9, type=int,  help='output tree format, default is 9')
    args = vars(replace_clade_parser.parse_args())
    replace_clade(args)



import os
import argparse


parse_reltime_usage = '''
==================== parse_reltime example commands ====================

TreeSAK parse_reltime -i RelTime.txt -n dbscc_lca.txt -o dbscc_age.txt

========================================================================
'''

def sep_path_basename_ext(file_in):

    f_path, f_name = os.path.split(file_in)
    if f_path == '':
        f_path = '.'
    f_base, f_ext = os.path.splitext(f_name)

    return f_name, f_path, f_base, f_ext[1:]


def get_lca(reltime_txt, leaf_1_name, leaf_2_name):

    leaf_set = set()
    child_to_parent_dict = dict()
    id_to_name_dict = dict()
    name_to_id_dict = dict()
    for each_line in open(reltime_txt):
        if not each_line.startswith('NodeLabel'):
            each_line_split = each_line.strip().split('\t')
            each_line_split = [i.strip() for i in each_line_split]
            if len(each_line_split) > 1:
                node_name = each_line_split[0].replace(' ', '_')
                node_id = each_line_split[1]
                des1 = each_line_split[2]
                des2 = each_line_split[3]
                id_to_name_dict[node_id] = node_name
                name_to_id_dict[node_name] = node_id
                child_to_parent_dict[des1] = node_id
                child_to_parent_dict[des2] = node_id
                if (des1 == '-') and (des2 == '-'):
                    leaf_set.add(node_id)

    leaf_to_lineage_dict = dict()
    for leaf in sorted([i for i in leaf_set]):
        original_leaf = leaf
        lineage_list = [leaf]
        while leaf in child_to_parent_dict:
            leaf_p = child_to_parent_dict[leaf]
            lineage_list.append(leaf_p)
            leaf = leaf_p
        leaf_to_lineage_dict[original_leaf] = lineage_list

    leaf_1_id     = name_to_id_dict[leaf_1_name]
    leaf_2_id     = name_to_id_dict[leaf_2_name]
    leaf_1_linage = leaf_to_lineage_dict[leaf_1_id]
    leaf_2_linage = leaf_to_lineage_dict[leaf_2_id]

    lca = ''
    for each_p in leaf_1_linage[::-1]:
        if each_p in leaf_2_linage:
            lca = each_p
    return lca


def parse_reltime(args):

    reltime_txt          = args['i']
    interested_nodes_txt = args['n']
    op_txt               = args['o']

    f_name, f_path, f_base, f_ext = sep_path_basename_ext(op_txt)
    op_txt_all_info = '%s/%s_all_info.%s' % (f_path,f_base, f_ext)

    lca_to_leaves_dict = dict()
    interested_node_desc_dict = dict()
    for interested_node in open(interested_nodes_txt):
        interested_node_split = interested_node.strip().split('\t')
        paired_leaves = interested_node_split[0]
        interested_node_desc = paired_leaves
        if len(interested_node_split) > 1:
            interested_node_desc = interested_node_split[1]
        interested_node_desc_dict[paired_leaves] = interested_node_desc
        leaf_1 = paired_leaves.split(',')[0]
        leaf_2 = paired_leaves.split(',')[1]
        lca_id = get_lca(reltime_txt, leaf_1, leaf_2)
        lca_to_leaves_dict[lca_id] = paired_leaves.strip()

    op_txt_all_info_handle = open(op_txt_all_info, 'w')
    line_num_index = 0
    for each_line in open(reltime_txt):
        each_line_split = each_line.strip().split('\t')
        each_line_split = [i.strip() for i in each_line_split]
        if line_num_index == 0:
            op_txt_all_info_handle.write('Leaves\tDescription\t%s\n' % ('\t'.join(each_line_split)))
        else:
            if len(each_line_split) > 1:
                node_id = each_line_split[1]
                if node_id in lca_to_leaves_dict:
                    node_id = each_line_split[1]
                    corresponding_leaves = lca_to_leaves_dict[node_id]
                    interested_node_desc = interested_node_desc_dict[corresponding_leaves]
                    op_txt_all_info_handle.write('%s\t%s\t%s\n' % (corresponding_leaves, interested_node_desc, '\t'.join(each_line_split)))
        line_num_index += 1
    op_txt_all_info_handle.close()

    op_txt_handle = open(op_txt, 'w')
    op_txt_handle.write('Node\tDivTime\tCI_Lower\tCI_Upper\n')
    line_num_index = 0
    for each_line in open(op_txt_all_info):
        if line_num_index > 0:
            each_line_split = each_line.strip().split('\t')
            desc = each_line_split[1]
            div_time = each_line_split[9]
            ci_lower = each_line_split[10]
            ci_upper = each_line_split[11]
            op_txt_handle.write('%s\t%s\t%s\t%s\n' % (desc, div_time, ci_lower, ci_upper))
        line_num_index += 1
    op_txt_handle.close()


if __name__ == '__main__':

    parse_reltime_parser = argparse.ArgumentParser()
    parse_reltime_parser.add_argument('-i',  required=True, help='reltime output file')
    parse_reltime_parser.add_argument('-n',  required=True, help='interested node txt')
    parse_reltime_parser.add_argument('-o',  required=True, help='output txt file')
    args = vars(parse_reltime_parser.parse_args())
    parse_reltime(args)


'''

cd /Users/songweizhi/Desktop
python3 /Users/songweizhi/PycharmProjects/TreeSAK/TreeSAK/parse_reltime.py -i /Users/songweizhi/Desktop/Sponge_r220/6_dating/RelTime/topo2_p30_RelTime_JTT_Gamma4/topo2_p30_RelTime_Gamma4.txt -n yang_7.txt -o dbscc_age.txt

cd /Users/songweizhi/Desktop
TreeSAK parse_reltime -i /Users/songweizhi/Desktop/Sponge_r220/6_dating/RelTime/topo2_p30_RelTime_JTT_Gamma4/topo2_p30_RelTime_Gamma4.txt -n yang_7.txt -o dbscc_age.txt

'''



import os
import argparse
from ete3 import Tree


format_leaf_name_usage = '''
========= FLN (Format Leaf Name) example commands =========

BioSAK FLN -i input.tree -o output.tree -s2u -nsqm -ndqm
BioSAK FLN -i input.tree -o output.tree -ns -nsqm

===========================================================
'''


def format_leaf_name(args):

    tree_file_in             = args['i']
    tree_format              = args['fmt']
    tree_file_out            = args['o']
    no_space                 = args['ns']
    space_to_underscore      = args['s2u']
    no_single_quotation_mark = args['nsqm']
    no_double_quotation_mark = args['ndqm']

    if os.path.isfile(tree_file_in) is False:
        print('Tree file not found, program exited!')
        exit()

    if (no_space is True) and (space_to_underscore is True):
        print('Two actions (-ns and -s2u) specified to spaces in tree leaves, program exited!')
        exit()

    t = Tree(tree_file_in, format=tree_format)

    # get rename dict
    mag_rename_dict = dict()
    for leaf in t:
        leaf_name = leaf.name
        if space_to_underscore is True:
            leaf_name = leaf_name.replace(' ', '_')
        if no_space is True:
            leaf_name = leaf_name.replace(' ', '')
        if no_single_quotation_mark is True:
            leaf_name = leaf_name.replace("'", '')
        if no_double_quotation_mark is True:
            leaf_name = leaf_name.replace('"', '')
        mag_rename_dict[leaf.name] = leaf_name

    for leaf in t:
        leaf_name = leaf.name
        leaf_name_new = mag_rename_dict[leaf_name]
        leaf.name = leaf_name_new
    t.write(format=tree_format, outfile=tree_file_out)

    print('Done!')


if __name__ == '__main__':

    format_leaf_name_parser = argparse.ArgumentParser()
    format_leaf_name_parser.add_argument('-i',                  required=True,                          help='input tree')
    format_leaf_name_parser.add_argument('-fmt',                required=False, default=1,              help='tree format, default: 1')
    format_leaf_name_parser.add_argument('-o',                  required=True,                          help='output tree')
    format_leaf_name_parser.add_argument('-s2u',                required=False, action="store_true",    help='change space in tree leaves to underscore')
    format_leaf_name_parser.add_argument('-ns',                 required=False, action="store_true",    help='remove space from leaf names')
    format_leaf_name_parser.add_argument('-nsqm',               required=False, action="store_true",    help='remove single quotation marks from leaf names')
    format_leaf_name_parser.add_argument('-ndqm',               required=False, action="store_true",    help='remove double quotation marks from leaf names')
    args = vars(format_leaf_name_parser.parse_args())
    format_leaf_name(args)



import os
import argparse


def sep_path_basename_ext(file_in):

    # separate path and file name
    f_path, file_name = os.path.split(file_in)
    if f_path == '':
        f_path = '.'

    # separate file basename and extension
    f_base, f_ext = os.path.splitext(file_name)

    return f_path, f_base, f_ext


SingleAleHGT_usage = '''
============================================ SingleAleHGT example commands ============================================

TreeSAK SingleAleHGT -i concatenated.fasta -s genome.treefile -fc 0.3 -c genome_taxon.txt -color phylum_color.txt -api S1kZZuDHc0d5M7J5vLnUNQ -t 9 -f -o demo_SingleAleHGT_wd

=======================================================================================================================
'''

def SingleAleHGT(args):

    faa_in                      = args['faa']
    msa_in                      = args['msa']
    op_dir                      = args['o']
    genome_tree_file_rooted     = args['s']
    API_key                     = args['api']
    hgt_freq_cutoff             = args['fc']
    ar_phylum_color_code_txt    = args['color']
    genome_taxon_txt            = args['c']
    force_overwrite             = args['f']
    trim_msa                    = args['trim']
    docker_image                = args['docker']
    num_threads                 = args['t']

    ######################################## check input files #######################################

    # if docker_image is True, check if docker is activated
    if (faa_in is not None) and (msa_in is None):
        f_path, f_base, f_ext = sep_path_basename_ext(faa_in)
    elif (faa_in is None) and (msa_in is not None):
        f_path, f_base, f_ext = sep_path_basename_ext(msa_in)
    else:
        print('Please specify either -faa or -msa, program exited!')
        exit()

    ######################################## define file name ########################################

    ale1_op_dir = '%s/ALE1_op_dir'      % op_dir
    ale2_op_dir = '%s/ALE2_op_dir'      % op_dir
    ale4_op_dir = '%s/ALE4_op_dir'      % op_dir
    log_txt     = '%s/log.txt'          % op_dir
    msa_file    = '%s/%s.aln'           % (ale1_op_dir, f_base)
    msa_trimmed = '%s/%s_trimmed.aln'   % (ale1_op_dir, f_base)
    tree_prefix = '%s/%s'               % (ale1_op_dir, f_base)

    ###################################### create output folder ######################################

    if os.path.isdir(op_dir) is True:
        if force_overwrite is True:
            os.system('rm -r %s' % op_dir)
        else:
            print('%s exist, program exited!' % op_dir)
            exit()
    os.mkdir(op_dir)
    os.mkdir(ale1_op_dir)

    ##################################################################################################

    # run mafft-einsi
    if (faa_in is not None) and (msa_in is None):
        mafft_cmd = 'mafft-einsi --thread %s --quiet %s > %s' % (num_threads, faa_in, msa_file)

        with open(log_txt, 'a') as log_txt_handle:
            log_txt_handle.write(mafft_cmd + '\n')
        os.system(mafft_cmd)
        msa_file_for_next_step = msa_file
    else:
        msa_file_for_next_step = msa_in

    # run trimal
    if trim_msa is True:
        trimal_cmd = 'trimal -in %s -out %s -automated1' % (msa_file_for_next_step, msa_trimmed)
        with open(log_txt, 'a') as log_txt_handle:
            log_txt_handle.write(trimal_cmd + '\n')
        os.system(trimal_cmd)
        iqtree2_cmd = 'iqtree2 -m LG+G+I -bb 1000 --wbtl -nt %s -s %s -pre %s' % (num_threads, msa_trimmed, tree_prefix)
        with open(log_txt, 'a') as log_txt_handle:
            log_txt_handle.write(iqtree2_cmd + '\n')
        os.system(iqtree2_cmd)
    else:
        iqtree2_cmd = 'iqtree2 -m LG+G+I -bb 1000 --wbtl -nt %s -s %s -pre %s' % (num_threads, msa_file_for_next_step, tree_prefix)
        with open(log_txt, 'a') as log_txt_handle:
            log_txt_handle.write(iqtree2_cmd + '\n')
        os.system(iqtree2_cmd)

    # run ALE2
    ale2_cmd = 'TreeSAK ALE2 -i %s -s %s -t %s -f -runALE -docker %s -o %s' % (ale1_op_dir, genome_tree_file_rooted, num_threads, docker_image, ale2_op_dir)
    with open(log_txt, 'a') as log_txt_handle:
        log_txt_handle.write(ale2_cmd + '\n')
    os.system(ale2_cmd)

    # run ALE4
    ale4_cmd = 'TreeSAK ALE4 -i1 %s -i2 %s -c %s -color %s -o %s -fc %s -f -api %s' % (ale1_op_dir, ale2_op_dir, genome_taxon_txt, ar_phylum_color_code_txt, ale4_op_dir, hgt_freq_cutoff, API_key)
    with open(log_txt, 'a') as log_txt_handle:
        log_txt_handle.write(ale4_cmd + '\n')
    os.system(ale4_cmd)


if __name__ == '__main__':

    SingleAleHGT_parser = argparse.ArgumentParser()
    SingleAleHGT_parser.add_argument('-faa',    required=False, default=None,               help='input aa file, e.g., OMA0001.faa')
    SingleAleHGT_parser.add_argument('-msa',    required=False, default=None,               help='input MSA file, e.g., OMA0001.aln')
    SingleAleHGT_parser.add_argument('-o',      required=True,                              help='output dir, e.g., SingleAleHGT_wd')
    SingleAleHGT_parser.add_argument('-s',      required=True,                              help='rooted species tree')
    SingleAleHGT_parser.add_argument('-c',      required=True,                              help='genome_taxon, GTDB format')
    SingleAleHGT_parser.add_argument('-color',  required=True,                              help='phylum color code')
    SingleAleHGT_parser.add_argument('-fc',     required=False, type=float, default=0.5,    help='hgt_freq_cutoff, default: 0.5')
    SingleAleHGT_parser.add_argument('-mld',    required=False, type=int, default=5,        help='donor_node_min_leaf_num, default: 5')
    SingleAleHGT_parser.add_argument('-mlr',    required=False, type=int, default=5,        help='recipient_node_min_leaf_num, default: 5')
    SingleAleHGT_parser.add_argument('-trim',   required=False, action="store_true",        help='trim MSA')
    SingleAleHGT_parser.add_argument('-docker', required=False, default=None,               help='Docker image, if ALE was installed with Docker, e.g., gregmich/alesuite_new')
    SingleAleHGT_parser.add_argument('-itol',   required=False, default='batch_access_tmp', help='iTOL project_name, default: batch_access_tmp')
    SingleAleHGT_parser.add_argument('-api',    required=True,                              help='iTOL API key')
    SingleAleHGT_parser.add_argument('-t',      required=False, type=int, default=6,        help='number of threads, default: 6')
    SingleAleHGT_parser.add_argument('-f',      required=False, action="store_true",        help='force overwrite')
    args = vars(SingleAleHGT_parser.parse_args())
    SingleAleHGT(args)


'''

cd /Users/songweizhi/Desktop/DateArTree/01_HGT_ALE_with_OMA/ALE1_op_dir_OMA05484_OMA07484_trimmed
trimal -in ../ALE1_op_dir_OMA05484_OMA07484/concatenated.fasta -out concatenated.fasta -automated1
iqtree2 -m LG+G+I -bb 1000 --wbtl -nt 10 -s concatenated.fasta -pre OMA05484_OMA07484
cd /Users/songweizhi/Desktop/DateArTree/01_HGT_ALE_with_OMA
TreeSAK ALE2 -i ALE1_op_dir_OMA05484_OMA07484_trimmed -s genome_tree.newick -t 10 -f -runALE -docker gregmich/alesuite_new -o ALE2_op_dir_OMA05484_OMA07484_trimmed
TreeSAK ALE4 -i1 ALE1_op_dir_OMA05484_OMA07484_trimmed -i2 ALE2_op_dir_OMA05484_OMA07484_trimmed -c genome_taxon.txt -color phylum_color.txt -o ALE4_op_dir_OMA05484_OMA07484_trimmed_0.01 -fc 0.01 -f -api S1kZZuDHc0d5M7J5vLnUNQ

cd /Users/songweizhi/Desktop/DateArTree/01_HGT_ALE_with_OMA
/usr/local/bin/python3.7 /Users/songweizhi/PycharmProjects/TreeSAK/TreeSAK/SingleAleHGT.py -msa ALE1_op_dir_OMA05484_OMA07484_trimmed/concatenated.fasta -s genome_tree_rooted_noEU.treefile -fc 0.3 -c genome_taxon.txt -color phylum_color.txt -api S1kZZuDHc0d5M7J5vLnUNQ -t 9 -f -o demo_SingleAleHGT_wd -trim

cd /Users/songweizhi/Desktop/DateArTree/01_HGT_ALE_with_OMA/demo_SingleAleHGT_wd
TreeSAK ALE2 -i ALE1_op_dir -s ../genome_tree.newick -t 10 -f -runALE -docker gregmich/alesuite_new -o ALE2_op_dir
TreeSAK ALE4 -i1 ALE1_op_dir_OMA05484_OMA07484_trimmed -i2 ALE2_op_dir_OMA05484_OMA07484_trimmed -c genome_taxon.txt -color phylum_color.txt -o ALE4_op_dir_OMA05484_OMA07484_trimmed_0.01 -fc 0.01 -f -api S1kZZuDHc0d5M7J5vLnUNQ

/usr/local/bin/python3.7 /Users/songweizhi/PycharmProjects/TreeSAK/TreeSAK/SingleAleHGT.py -o demo_SingleAleHGT_wd -msa ALE1_op_dir/OMA15312.aln -s genome_tree_rooted_noEU.treefile -fc 0.3 -c genome_taxon.txt -color phylum_color.txt -api S1kZZuDHc0d5M7J5vLnUNQ -t 10 -f -trim -docker gregmich/alesuite_new
/usr/local/bin/python3.7 /Users/songweizhi/PycharmProjects/TreeSAK/TreeSAK/SingleAleHGT.py -o OMA01402_ALE_HGT_wd -msa ALE1_op_dir/OMA01402.aln -s genome_tree_rooted_noEU.treefile -fc 0.3 -c genome_taxon.txt -color phylum_color.txt -api S1kZZuDHc0d5M7J5vLnUNQ -t 10 -f -trim -docker gregmich/alesuite_new
/usr/local/bin/python3.7 /Users/songweizhi/PycharmProjects/TreeSAK/TreeSAK/SingleAleHGT.py -o OMA01402_ALE_HGT_wd_no_trim -msa ALE1_op_dir/OMA01402.aln -s genome_tree_rooted_noEU.treefile -fc 0.3 -c genome_taxon.txt -color phylum_color.txt -api S1kZZuDHc0d5M7J5vLnUNQ -t 10 -f -docker gregmich/alesuite_new

'''



import os
import argparse


AssessPB_usage = '''
====================== AssessPB example commands ======================

# Dependency: bpcomp and tracecomp (from PhyloBayes-MPI)

export OMPI_MCA_btl=^openib

TreeSAK AssessPB -c1 c1dir/c1 -c2 c2dir/c2
TreeSAK AssessPB -c1 c1dir/c1 -c2 c2dir/c2 -c3 c3dir/c3
TreeSAK AssessPB -c1 c1dir/c1 -c2 c2dir/c2 -c3 c3dir/c3 -c4 c4dir/c4
TreeSAK AssessPB -cdir chain_dir

# This is a wrapper for: 
bpcomp -x 1000 10 c1 c2
bpcomp -x 1000 10 c1 c2 c3 c4
tracecomp -x 1000 c1 c2
tracecomp -x 1000 c1 c2 c3 c4

=======================================================================
'''


def compare2chains(chain_1, chain_2, chain_3, chain_4, burn_in, sample_interval, with_bpcomp, with_tracecomp, op_dir, cmd_txt):

    # bpcomp:    -x <burnin> [<every> <until>]. default burnin = 10 percent of the chain
    # tracecomp: -x <burnin> [<every> <until>]. default burnin = 20 percent of the chain

    bpcomp_cmd    = 'bpcomp -o %s/bpcomp -x %s %s %s %s'    % (op_dir, burn_in, sample_interval, chain_1, chain_2)
    tracecomp_cmd = 'tracecomp -o %s/tracecomp -x %s %s %s' % (op_dir, burn_in, chain_1, chain_2)

    if (chain_3 is not None) and (chain_4 is None):
        bpcomp_cmd    = 'bpcomp -o %s/bpcomp -x %s %s %s %s %s'    % (op_dir, burn_in, sample_interval, chain_1, chain_2, chain_3)
        tracecomp_cmd = 'tracecomp -o %s/tracecomp -x %s %s %s %s' % (op_dir, burn_in, chain_1, chain_2, chain_3)

    if (chain_3 is not None) and (chain_4 is not None):
        bpcomp_cmd    = 'bpcomp -o %s/bpcomp -x %s %s %s %s %s %s'    % (op_dir, burn_in, sample_interval, chain_1, chain_2, chain_3, chain_4)
        tracecomp_cmd = 'tracecomp -o %s/tracecomp -x %s %s %s %s %s' % (op_dir, burn_in, chain_1, chain_2, chain_3, chain_4)

    # write out commands
    cmd_txt_handle = open(cmd_txt, 'a')
    cmd_txt_handle.write(bpcomp_cmd + '\n')
    cmd_txt_handle.write(tracecomp_cmd + '\n')
    cmd_txt_handle.close()

    # execute commands
    if with_bpcomp is True:
        print()
        print('====================== bpcomp ======================')
        print()
        print(bpcomp_cmd)
        os.system(bpcomp_cmd)
        print('Guideline')
        print('1. maxdiﬀ < 0.1: good run.')
        print('2. maxdiﬀ < 0.3: acceptable: gives a good qualitative picture of the posterior consensus.')
        print('3. 0.3 < maxdiﬀ < 1: the sample is not yet suﬃciently large and have not converged, but on right track.')
        print('4. if maxdiﬀ = 1 even after 10,000 points: at least one run stuck in a local maximum.')
        print()

    if with_tracecomp is True:
        print('==================== tracecomp ====================')
        print()
        print(tracecomp_cmd)
        print()
        os.system(tracecomp_cmd)
        print()
        print('Guideline')
        print('1. rel diﬀ < 0.1 and minimum eﬀective size > 300: good run.')
        print('2. rel diﬀ < 0.3 and minimum eﬀective size > 50: acceptable run.')
        print()

    print('====================================================')


def AssessPB(args):

    chain_1         = args['c1']
    chain_2         = args['c2']
    chain_3         = args['c3']
    chain_4         = args['c4']
    chain_dir       = args['cdir']
    burn_in         = args['bi']
    sample_interval = args['si']
    op_dir          = args['o']
    force_overwrite = args['f']
    with_bpcomp     = True
    with_tracecomp  = True

    cmd_txt         = '%s/cmds.txt' % op_dir

    # create output dir
    if os.path.isdir(op_dir) is True:
        if force_overwrite is True:
            os.system('rm -r %s' % op_dir)
        else:
            print('output folder already exist, program exited!')
            exit()
    os.system('mkdir %s' % op_dir)

    if (chain_1 is not None) and (chain_2 is not None) and (chain_dir is None):
        compare2chains(chain_1, chain_2, chain_3, chain_4, burn_in, sample_interval, with_bpcomp, with_tracecomp, op_dir, cmd_txt)

    elif (chain_1 is None) and (chain_2 is None) and (chain_dir is not None):
        print('Compare multiple chains')
        print('Function to be added!')
        print('Program exited!')

    else:
        print('Please compare either no more than four chains (specified by -c1, -c2, -c3 and -c4) or multiple chains provided within -cdir')
        print('Program exited!')
        exit()


if __name__ == '__main__':

    AssessPB_parser = argparse.ArgumentParser()
    AssessPB_parser.add_argument('-c1',     required=False, default=None,           help='chain 1')
    AssessPB_parser.add_argument('-c2',     required=False, default=None,           help='chain 2')
    AssessPB_parser.add_argument('-c3',     required=False, default=None,           help='chain 3')
    AssessPB_parser.add_argument('-c4',     required=False, default=None,           help='chain 4')
    AssessPB_parser.add_argument('-cdir',   required=False, default=None,           help='chain folder')
    AssessPB_parser.add_argument('-bi',     required=False, default=1000,           help='burn-in, default: 1000')
    AssessPB_parser.add_argument('-si',     required=False, default=10,             help='sample interval, default: 10')
    AssessPB_parser.add_argument('-o',      required=True, default=None,            help='output directory')
    AssessPB_parser.add_argument('-f',      required=False, action="store_true",    help='force overwrite')
    args = vars(AssessPB_parser.parse_args())
    AssessPB(args)



import io
import pandas as pd
from tqdm import tqdm
from ete3 import Tree
from glob import glob
from os.path import *
import plotly.express as px
import plotly.graph_objects as go
import plotly.figure_factory as ff


def read_mcmc(mcmc, all_col=False):
    if type(mcmc) != str:
        return mcmc
    if all_col:
        mcmc_df = pd.read_csv(mcmc, sep='\t', index_col=0)
    else:
        f1 = open(mcmc)
        header = [_ for _ in next(f1).strip().split('\t')]
        r_header = [_ for _ in header if not _.startswith('r_g')]
        # normally it need to iterate rows and ignore the columns representing rates
        text = '\t'.join(r_header) + '\n'
        r_header = set(r_header)
        for row in f1:
            text += '\t'.join([r for r, h in zip(row.strip().split('\t'), header) if h in r_header]) + '\n'
        mcmc_df = pd.read_csv(io.StringIO(text), sep='\t', index_col=0)
    return mcmc_df


def get_node_name_from_log(f):
    # f should be the *.log file
    rows = open(f).read().split('\n')
    idx = [_ for _, r in enumerate(rows) if r == 'Species tree']
    if not idx:
        print("prior not complete")
        return
    idx = idx[0]
    start_idx = idx + 3
    end_idx = 0
    for _ in range(idx, 100000):
        if rows[_] == '':
            end_idx = _
            break
    tree_idx1 = end_idx + 1
    tree_idx2 = end_idx + 2
    # find the index
    n2father = {}
    for i in range(start_idx, end_idx):
        row = [_ for _ in rows[i].split(' ') if _]
        father, n, name = row[0], row[1], row[2]
        n2father[name if len(row) == 4 else n] = father

    t = Tree(rows[tree_idx2], format=8)
    for l in t.traverse('postorder'):
        if l.up is None:
            break
        if not l.up.name:
            l.up.name = n2father[l.name]
    return t


indir                   = '/Users/songweizhi/Desktop/DateArTree/plot_distruibution/stepwise'
tree_dir                = '/Users/songweizhi/Desktop/DateArTree/plot_distruibution/treefile_dir'
plot_dir                = '/Users/songweizhi/Desktop/DateArTree/plot_distruibution'

gene_names              = ['M24', 'COG25']
M24_gene_list           = ['MitoCOG0043', 'MitoCOG0040', 'MitoCOG0055', 'MitoCOG0052', 'MitoCOG0053', 'MitoCOG0133', 'MitoCOG0008', 'MitoCOG0009', 'MitoCOG0027', 'MitoCOG0031', 'MitoCOG0030', 'MitoCOG0001', 'MitoCOG0003', 'MitoCOG0012', 'MitoCOG0010', 'MitoCOG0004', 'MitoCOG0005', 'MitoCOG0011', 'MitoCOG0039', 'MitoCOG0060', 'MitoCOG0071', 'MitoCOG0059', 'MitoCOG0067', 'MitoCOG0066']
COG25_gene_list         = ['223163', '223176', '223175', '223607', '223159', '223165', '223170', '223164', '223158', '223172', '223128', '223665', '223275', '223328', '223280', '223127', '223279', '273102', '223130', '223181', '223180', '223168', '223178', '223596', '223556']


setname2genes = dict()
setname2genes['M24']   = M24_gene_list
setname2genes['COG25'] = COG25_gene_list


gene2num = {}
gene2dl = {}
for gene_id in (M24_gene_list + COG25_gene_list):
    pwd_tree_file  = '%s/%s.treefile' % (tree_dir, gene_id)
    pwd_iqtree_log = '%s/%s.iqtree'   % (tree_dir, gene_id)
    rows = open(pwd_iqtree_log).read().strip().split("\n")
    idx = [idx for idx, v in enumerate(rows) if "deltaL  bp-RELL" in v][0]
    r1, r2 = rows[idx + 2], rows[idx + 3]
    r1 = [_ for _ in r1.strip().split(" ") if _]
    r2 = [_ for _ in r2.strip().split(" ") if _]
    if r2[2] == "0":
        gene2dl[gene_id] = float(r1[2])
    else:
        gene2dl[gene_id] = float(r2[2])
    gene2num[gene_id] = len(Tree(pwd_tree_file).get_leaf_names())

# plot 1
for setname, genes in setname2genes.items():
    dl_list = [gene2dl[_] for _ in genes]
    dl_list = sorted(dl_list, reverse=True)
    fig = go.Figure()
    fig.add_bar(y=dl_list)
    fig.update_layout(title_text=setname,title_x=0.5,title_y=1,width=700,height=100,template='simple_white',
                     margin_b=10,margin_l=10,margin_r=10,margin_t=10)
    fig.write_image('%s/Plot_1_%s.pdf' % (plot_dir, setname))

for gene_set in gene_names:
    for _model in ['LG']:  # C60
        t = []
        for f in glob(f'{indir}/{gene_set}/r*/1pf_{_model}/mcmctree/mcmc.txt'):
            if exists(f.replace('mcmc.txt', 'FigTree.tre')):
                t.append((f.split('/')[-4] + ' MCMC', f))
        t = sorted(t, key=lambda x: int(x[0].split(' ')[0][1:]))

        dfs = []
        targets = []
        for cal, mcmc in tqdm(t):
            tre = get_node_name_from_log(mcmc.replace('mcmc.txt','03_mcmctree.log'))
            df  = read_mcmc(mcmc)
            try:
                df = df.sample(5000)
            except:
                print(mcmc)
            for lca, name in [('GCA_001828545.1,GCA_005524015.1', 'Anammox'), ('GCA_013697045.1,GCA_002356115.1', 'Gamma-AOB'),
                              ('GCA_001772005.1,GCA_013521015.1', 'Beta-AOB'), ('GCA_017879665.1,GCA_013140535.1', 'Comammox'),
                              ('Acanthamoeba_castellanii,Andalucia_godoyi', 'Euk'), ('Andalucia_godoyi,Ostreococcus_tauri', 'Euk'),
                              ('Cyanophora_paradoxa,NC_002186.1', 'Euk')]:
                try:
                    n = tre.get_common_ancestor(lca.split(',')).name
                    targets.append(str(n))
                    n = 't_n' + str(n)
                    times = df[[n]]
                except:
                    continue

                times.columns = ['time']
                times.loc[:, 'group name'] = name
                times.loc[:, 'cal'] = cal
                dfs.append(times)

        # plot 2
        _df = pd.concat(dfs, axis=0)
        g2color = {"Gamma-AOB": "#78fce0", "Beta-AOB": "#956bb4", "Comammox": "#edc21a", "Anammox": "#ff8000"}
        _df = _df.loc[_df["group name"].isin(list(g2color)), :]
        _fig = px.violin( _df, y="cal", x="time", color="group name", color_discrete_map=g2color, points=False, orientation="h")
        _fig.update_traces(side="positive", fillcolor='rgba(0,0,0,0)', width=1.8)
        _fig.update_traces(showlegend=False)
        num_y = len(_df["cal"].unique())
        _fig.layout.template = "simple_white"
        _fig.layout.width = 700
        _fig.layout.height = 750
        _fig.update_xaxes(range=[40, 0])
        _fig.update_layout(margin_t=10, title_text=f'{gene_set} {_model}', title_x=0.5)
        _fig.write_image(f'{plot_dir}/Plot_2_{gene_set}_gradient_{_model}.pdf')

        # plot 3
        xs = []
        ys = []
        for ng, subdf in sorted(_df.groupby('cal'),key=lambda x: int(x[0].split(' ')[0].replace('r', ''))):
            t1 = subdf.loc[subdf['group name'] == 'Gamma-AOB', 'time'].median()
            t2 = subdf.loc[subdf['group name'] == 'Anammox', 'time'].median()
            deltaT = t2-t1
            ys.append(deltaT)
            xs.append(int(ng.split(' ')[0].replace('r', '')))
        fig = go.Figure()
        fig.add_scatter(x=xs, y=ys, mode='markers+lines', showlegend=False)
        fig.update_layout(width=300, height=300, margin_t=30, margin_l=10, margin_b=10, margin_r=10,
                          template='simple_white', title_text=f'{gene_set} {_model}', title_x=0.5)
        fig.write_image('%s/Plot_3_%s_%s.pdf' % (plot_dir, gene_set, _model))




import argparse
from Bio import SeqIO


gap_stats_usage = '''
======= gap_stats example commands =======

TreeSAK gap_stats -i msa.fasta

==========================================
'''


def gap_stats(args):

    msa_in_fa = args['i']

    gap_pct_dict = dict()
    for each_seq in SeqIO.parse(msa_in_fa, 'fasta'):
        seq_id = each_seq.id
        seq_str = str(each_seq.seq)
        gap_pct = seq_str.count('-')*100/len(seq_str)
        gap_pct = float("{0:.2f}".format(gap_pct))
        gap_pct_dict[seq_id] = gap_pct

    gap_pct_sorted = sorted(gap_pct_dict.items(), key=lambda x: x[1])

    print('Sequence\tGap')
    for each_seq in gap_pct_sorted:
        print('%s\t%s' % (each_seq[0], each_seq[1]))


if __name__ == '__main__':

    gap_stats_parser = argparse.ArgumentParser()
    gap_stats_parser.add_argument('-i', required=True, help='MSA in fasta format')
    args = vars(gap_stats_parser.parse_args())
    gap_stats(args)



from ete3 import Tree
from os.path import join, dirname, exists


def get_topology_without_alpha(st, two_nodes_1, two_nodes_2):

    _st = st.copy()
    n_replaced = _st.get_common_ancestor(two_nodes_1)

    if len(n_replaced.get_leaf_names()) == 10:
        n_p = n_replaced.up
        n_p_p = n_p.up   # All proteobacteria
        n_p.remove_child(n_replaced)
        return _st, n_p_p, n_p
    else:
        beta_gamma = _st.get_common_ancestor(two_nodes_2)
        for _ in n_replaced.children[::]:
            n_replaced.remove_child(_)
        n_replaced.add_child(beta_gamma)
        return _st, all_pro, n_replaced


def read_tree(in_tree, format=None):

    if isinstance(in_tree, str) and exists(in_tree):
        if format=='auto':
            for f in [0,1,2,3,4,5]:
                try:
                    t = Tree(in_tree, format=f)
                    return t
                except:
                    pass
        else:
            t = Tree(open(in_tree).read(), format=format)
    elif isinstance(in_tree, Tree):
        t = in_tree
    else:
        raise IOError('unknown input')
    return t


def erase_name(in_tree_file, format=0):

    t = read_tree(in_tree_file, format=format)
    for n in t.traverse():
        if not n.is_leaf():
            n.name = ''
    return t


def get_mito(dataset):

    euk_tree = Tree(euk_reference_tree, 8)
    euk_tree.prune(dataset)
    et = erase_name(euk_tree)
    return {"Mito": et}


########################################################################################################################

topo_in_txt = ''

intree                  = f'./dating/topology/mixture_models/deno100/phy/deno100.final_TP1.newick'
euk_reference_tree      = '/mnt/home-backup/thliao/AOB/analysis/update_calibrations/mito_dating/phylo/manual_topology/euk.tre'
euk_list_txt            = '/mnt/home-backup/thliao/AOB/analysis/update_calibrations/mito_dating/euk.list'
base_odir               = "./dating/topology"

Rickettsiales_lineage   = ['GCA_008189685.1', 'GCA_003015145.1']
Magneto_lineage         = ["GCA_000014865.1", "GCA_002109495.1"]
remaining_alpha         = ['GCA_000264455.2', 'GCA_002924445.1']
Holo                    = "GCA_000469665.2"
two_nodes_1             = ['GCA_000014865.1', 'GCA_000264455.2']
two_nodes_2             = ['GCA_018655245.1', 'GCA_002356115.1']

TP_dict                 = {"TP1": "((((Holo,other_alpha),Rick),Mito),Magneto);",
                           "TP2": "(((Holo,other_alpha),(Rick,Mito)),Magneto);",
                           "TP3": "((((Holo,Rick),other_alpha),Mito),Magneto);",
                           "TP4": "((((Mito,Rick),Holo),other_alpha),Magneto);"}

# copy from '/home-user/sswang/project/Mito/results/euk_tree/euk.tre'. The Fig S2A in wang 2021 NC
# rephrase Porphyra purpurea into Porphyra umbilicalis
# add Polysphondylium pallidum manually

'''

1. provide a input tree
2. provide a set of tree skeleton
3. use two leaves to determine a clade.

'''

########################################################################################################################


for tp_name, TP in TP_dict.items():

    print('%s\t%s' % (tp_name, TP))

    # get internal node to tree string dict

    st = Tree(intree, 8)
    _st, all_pro, n_replaced = get_topology_without_alpha(st.copy(), two_nodes_1, two_nodes_2)

    mito_usage = [_ for _ in open(euk_list_txt).read().split('\n')]
    m_dict = get_mito(mito_usage)

    nodes_dict = {"Rick"        : st.get_common_ancestor(Rickettsiales_lineage),
                  "other_alpha" : st.get_common_ancestor(remaining_alpha),
                  "Magneto"     : st.get_common_ancestor(Magneto_lineage),
                  "Holo"        : [_ for _ in st.traverse() if _.name == Holo][0],
                  "Mito"        : m_dict['Mito']}

    for k, n in nodes_dict.items():
        TP = TP.replace(k, n.write(format=3).strip(';'))
    n_replaced.add_child(Tree(TP, format=3))




import argparse
from Bio import SeqIO
from Bio import AlignIO


phy2fa_usage = '''
======= phy2fa example commands =======

TreeSAK phy2fa -i msa.phy -o msa.fa

=======================================
'''


def phy2fa(args):

    phylip_in   = args['i']
    fasta_out   = args['o']

    for aln in AlignIO.parse(phylip_in, "phylip"):
        print(aln)

    # alignments = list(AlignIO.parse(phylip_in, "phylip"))
    # print(alignments)
    # records = SeqIO.parse(phylip_in, "phylip")
    # count = SeqIO.write(records, fasta_out, "fasta")
    # print("Converted %i records" % count)


if __name__ == '__main__':

    # initialize the options parser
    phy2fa_parser = argparse.ArgumentParser()
    phy2fa_parser.add_argument('-i',      required=True,   help='input MSA in phylip format')
    phy2fa_parser.add_argument('-o',      required=True,   help='output MSA in fasta format')
    args = vars(phy2fa_parser.parse_args())
    phy2fa(args)



import os
import math
import argparse
from Bio import SeqIO


iTOL_msa_stats_usage = '''
========= iTOL_msa_stats example command =========

TreeSAK iTOL_msa_stats -i concatenated.phy.fasta

==================================================
'''


def sep_path_basename_ext(file_in):
    f_path, f_name = os.path.split(file_in)
    if f_path == '':
        f_path = '.'
    f_base, f_ext = os.path.splitext(f_name)
    f_ext = f_ext[1:]
    return f_name, f_path, f_base, f_ext


def iTOL_msa_stats(args):

    aln_file = args['i']

    _, aln_path, aln_base, _ = sep_path_basename_ext(aln_file)

    stats_txt      = '%s/%s_gap_pct.txt'         % (aln_path, aln_base)
    stats_txt_itol = '%s/%s_gap_pct_iTOL.txt'    % (aln_path, aln_base)

    max_gap_pct = 0
    stats_txt_handle = open(stats_txt, 'w')
    for each_seq in SeqIO.parse(aln_file, 'fasta'):
        seq_id = each_seq.id
        seq_seq = str(each_seq.seq)
        gap_pct = seq_seq.count('-')*100/len(seq_seq)
        gap_pct = float("{0:.2f}".format(gap_pct))
        if gap_pct > max_gap_pct:
            max_gap_pct = gap_pct
        stats_txt_handle.write('%s\t%s\n' % (seq_id, gap_pct))
    stats_txt_handle.close()

    max_scale_value = math.ceil(max_gap_pct/5) * 5
    gap_pct_itol_cmd = 'TreeSAK iTOL -SimpleBar -lv %s -scale 0-25-50-75-100 -lt Gap_Pecentage -o %s' % (stats_txt, stats_txt_itol)
    os.system(gap_pct_itol_cmd)


if __name__ == '__main__':

    iTOL_msa_stats_parser = argparse.ArgumentParser(usage=iTOL_msa_stats_usage)
    iTOL_msa_stats_parser.add_argument('-i', required=True, help='MSA file')
    args = vars(iTOL_msa_stats_parser.parse_args())
    iTOL_msa_stats(args)



import os
import argparse
import pandas as pd


ALE3_usage = '''
================= ALE3 example commands =================

TreeSAK ALE3 -2 ALE2_op_dir -o ALE3_op_dir_30 -f -c 30
TreeSAK ALE3 -2 ALE2_op_dir -o ALE3_op_dir_75 -f -c 75

# Needs the uml_rec files

=========================================================
'''


def ale_parser(rec_folder, SpeciesTreeRef_newick, TableInfo_tsv, TableEvents_tsv, GeneTrees_nwk):

    rec_files = [x for x in os.listdir(rec_folder) if x.endswith("uml_rec")]

    table_info = list()
    table_events = list()
    for rec_file in rec_files:
        with open(os.path.join(rec_folder, rec_file)) as f:
            fam = rec_file.replace(".ale.uml_rec", "")
            lines = f.readlines()
            stree = lines[2].strip()
            ll = lines[6].strip().split()[-1]
            dp, tp, lp = lines[8].strip().split("\t")[1:]
            n_reconciled_trees = int(lines[9].strip().split()[0])
            reconciled_trees = lines[11:n_reconciled_trees + 11]
            de, te, le, se = lines[11 + n_reconciled_trees + 1].split("\t")[1:]
            table = lines[11 + n_reconciled_trees + 3:]

        table_info.append((fam, ll, dp, tp, lp, de, te, le, se))
        table_events.append((fam, table))

    # write out SpeciesTreeRef.newick
    with open(SpeciesTreeRef_newick, "w") as f:
        f.write(stree.split("\t")[-1])

    # write out TableInfo.tsv
    with open(TableInfo_tsv, "w") as f:
        head = "\t".join(["Family", "LL", "Dp", "Tp", "Lp", "De", "Te", "Le", "Se"]) + "\n"
        f.write(head)
        for info in table_info:
            f.write("\t".join(info))

    # write out TableEvents.tsv
    with open(TableEvents_tsv, "w") as f:
        header = "Family\tBranchType\t" + table[0].replace("# of", "Branch")
        f.write(header)
        for fam, events in table_events:
            for b in events[1:]:
                f.write(fam + "\t" + b)

    # write out GeneTrees.nwk
    with open(GeneTrees_nwk, "w") as f:
        for t in reconciled_trees:
            f.write(t)


def get_verticality_and_transfer_propensity(TableEvents_tsv, verticality_txt, transfer_propensity_txt, fun_des_dict):

    df = pd.read_csv(TableEvents_tsv, sep="\t")
    dfb = df.groupby("Branch", as_index=False).sum()
    dff = df.groupby("Family").sum()

    dfb["Verticality"] = dfb["singletons"] / (dfb["singletons"] + dfb["Originations"] + dfb["Transfers"])
    dff["TransferPropensity"] = dff["Transfers"] / (dff["singletons"] + dff["Transfers"])

    verticality_dict = dfb.to_dict()['Verticality']
    transfer_propensity_dict = dff.to_dict()['TransferPropensity']

    with open(verticality_txt, 'w') as verticality_txt_handle:
        verticality_txt_handle.write('Branch\tVerticality\n')
        for each_key in sorted(list(verticality_dict.keys())):
            verticality_txt_handle.write('%s\t%s\n' % (each_key, verticality_dict[each_key]))

    with open(transfer_propensity_txt, 'w') as transfer_propensity_txt_handle:

        # write out header
        if len(fun_des_dict) == 0:
            transfer_propensity_txt_handle.write('OG\tTransfer_propensity\n')
        else:
            transfer_propensity_txt_handle.write('OG\tTransfer_propensity\tDescription\n')

        for each_key in sorted(list(transfer_propensity_dict.keys())):
            transfer_propensity = transfer_propensity_dict[each_key]
            transfer_propensity = float("{0:.3f}".format(transfer_propensity))
            each_key   = each_key.replace(('genome_tree.newick_'), '')
            each_key   = each_key.replace('.ufboot', '')
            if len(fun_des_dict) == 0:
                transfer_propensity_txt_handle.write('%s\t%s\n' % (each_key, transfer_propensity))
            else:
                transfer_propensity_txt_handle.write('%s\t%s\t%s\n' % (each_key, transfer_propensity, fun_des_dict.get(each_key, 'na')))


def ALE3(args):

    uml_rec_dir          = args['2']
    gene_presence_cutoff = args['c']
    fun_des_txt          = args['a']
    op_dir               = args['o']
    force_create_op_dir  = args['f']

    # read in fun_des_txt
    fun_des_dict = dict()
    if fun_des_txt is not None:
        if os.path.isfile(fun_des_txt) is True:
            for each_line in open(fun_des_txt):
                each_line_split = each_line.strip().split('\t')
                fun_des_dict[each_line_split[0]] = each_line_split[1]
        else:
            print('Specified "-a" not found, program exited!')
            exit()

    SpeciesTreeRef_newick   = '%s/SpeciesTreeRef.newick'    % op_dir
    TableInfo_tsv           = '%s/TableInfo.tsv'            % op_dir
    TableEvents_tsv         = '%s/TableEvents.tsv'          % op_dir
    GeneTrees_nwk           = '%s/GeneTrees.nwk'            % op_dir
    gene_content_dir        = '%s/GeneContent'              % op_dir
    gene_content_txt        = '%s/GeneContent.txt'          % op_dir
    verticality_txt         = '%s/Verticality.txt'          % op_dir
    transfer_propensity_txt = '%s/Transfer_propensity.txt'  % op_dir

    if os.path.isdir(op_dir) is True:
        if force_create_op_dir is True:
            os.system('rm -r %s' % op_dir)
        else:
            print('Output folder detected, program exited!')
            exit()
    os.system('mkdir %s' % op_dir)
    os.system('mkdir %s' % gene_content_dir)

    # parsing ALE2 outputs
    print('Parsing ALE2 outputs')
    ale_parser(uml_rec_dir, SpeciesTreeRef_newick, TableInfo_tsv, TableEvents_tsv, GeneTrees_nwk)

    # get_verticality_and_transfer_propensity
    print('Getting verticality and transfer propensity')
    get_verticality_and_transfer_propensity(TableEvents_tsv, verticality_txt, transfer_propensity_txt, fun_des_dict)

    # get genome content
    og_set = set()
    branch_to_og_dict = dict()
    col_index = {}
    for each_line in open(TableEvents_tsv):
        each_line_split = each_line.strip().split('\t')
        if each_line.startswith('Family'):
            col_index = {key: i for i, key in enumerate(each_line_split)}
        else:
            gene_family   = each_line_split[col_index['Family']]
            gene_family   = gene_family.replace(('genome_tree.newick_'), '')
            gene_family   = gene_family.replace('.ufboot', '')
            gene_branch   = each_line_split[col_index['Branch']]
            gene_presence = float(each_line_split[col_index['presence']])
            if gene_presence >= (gene_presence_cutoff/100):
                og_set.add(gene_family)
                if gene_branch not in branch_to_og_dict:
                    branch_to_og_dict[gene_branch] = set()
                branch_to_og_dict[gene_branch].add(gene_family)

    # write out gene content for each branch
    for each_branch in branch_to_og_dict:
        branch_gene_content = branch_to_og_dict[each_branch]

        current_gene_content_txt = '%s/%s.txt' % (gene_content_dir, each_branch)
        current_gene_content_txt_handle = open(current_gene_content_txt, 'w')
        for each_gene in sorted(list(branch_gene_content)):
            if len(fun_des_dict) == 0:
                current_gene_content_txt_handle.write('%s\n' % each_gene)
            else:
                current_gene_content_txt_handle.write('%s\t%s\n' % (each_gene, fun_des_dict.get(each_gene, 'na')))
        current_gene_content_txt_handle.close()

    og_list_sorted = sorted(list(og_set))

    gene_content_txt_handle = open(gene_content_txt, 'w')
    gene_content_txt_handle.write('Branch\t' + '\t'.join(og_list_sorted) + '\n')
    for each_gnm in sorted(list(branch_to_og_dict.keys())):
        og_pa_list = [each_gnm]
        for each_og in og_list_sorted:
            if each_og in branch_to_og_dict[each_gnm]:
                og_pa_list.append('1')
            else:
                og_pa_list.append('0')
        gene_content_txt_handle.write('\t'.join(og_pa_list) + '\n')
    gene_content_txt_handle.close()

    print('Protein families in GeneContent.txt: %s' % len(og_list_sorted))
    print('Genomes/branches in GeneContent.txt: %s' % len(branch_to_og_dict))


if __name__ == '__main__':

    ALE3_parser = argparse.ArgumentParser()
    ALE3_parser.add_argument('-2',   required=True,                             help='Folder with the uml_rec files')
    ALE3_parser.add_argument('-c',   required=False, type=float, default=75,   help='gene family presence cutoff in percentage, default: 75')
    ALE3_parser.add_argument('-a',   required=False, default=None,              help='OG functional description')
    ALE3_parser.add_argument('-o',   required=True,                             help='output dir')
    ALE3_parser.add_argument('-f',   required=False, action="store_true",       help='force overwrite')
    args = vars(ALE3_parser.parse_args())
    ALE3(args)



import argparse


deltall_usage = '''
========================= deltall example commands =========================

TreeSAK deltall -i nohup.out -o DeltaLL_op_summary.txt

# This script was wrote to parse the stdout of deltaLL.rb from Sishuo Wang

============================================================================
'''


def deltall(args):

    deltall_stdout_txt = args['i']
    summary_txt        = args['o']

    deltall_op_dict = dict()
    for each_line in open(deltall_stdout_txt):
        if not ((each_line.startswith('WARNING:')) or (each_line.startswith('awk:'))):
            each_line_split = each_line.strip().split('\t')
            marker_id = each_line_split[0]
            value = float(each_line_split[1])
            if marker_id not in deltall_op_dict:
                deltall_op_dict[marker_id] = [value]
            else:
                deltall_op_dict[marker_id].append(value)

    metric_1_dict = dict()
    metric_2_dict = dict()
    for each_marker in deltall_op_dict:
        metric_1_value = float("{0:.2f}".format(deltall_op_dict[each_marker][0]))
        metric_2_value = float("{0:.2f}".format(deltall_op_dict[each_marker][1]))
        metric_1_dict[each_marker] = metric_1_value
        metric_2_dict[each_marker] = metric_2_value

    metric_1_dict_sorted = {k: v for k, v in sorted(metric_1_dict.items(), key=lambda item: item[1])[::-1]}
    metric_2_dict_sorted = {k: v for k, v in sorted(metric_2_dict.items(), key=lambda item: item[1])}

    metric_1_score_dict = dict()
    metric_1_score = 1
    for each_marker_1 in metric_1_dict_sorted:
        metric_1_score_dict[each_marker_1] = metric_1_score
        metric_1_score += 1

    metric_2_score_dict = dict()
    metric_2_score = 1
    for each_marker_2 in metric_2_dict_sorted:
        metric_2_score_dict[each_marker_2] = metric_2_score
        metric_2_score += 1

    overall_score_dict = dict()
    for each_marker in deltall_op_dict:
        metric_score_1 = metric_1_score_dict[each_marker]
        metric_score_2 = metric_2_score_dict[each_marker]
        metric_score_overall = metric_score_1 + metric_score_2
        overall_score_dict[each_marker] = metric_score_overall

    overall_score_dict_sorted = {k: v for k, v in sorted(overall_score_dict.items(), key=lambda item: item[1])}

    summary_txt_handle = open(summary_txt, 'w')
    summary_txt_handle.write('Marker\tmetric1\tmetric1_score\tmetric2\tmetric2_score\toverall_score\n')
    for each_marker in overall_score_dict_sorted:
        metric_value_1 = metric_1_dict[each_marker]
        metric_value_2 = metric_2_dict[each_marker]
        metric_score_1 = metric_1_score_dict[each_marker]
        metric_score_2 = metric_2_score_dict[each_marker]
        metric_score_overall = overall_score_dict_sorted[each_marker]
        summary_txt_handle.write('%s\t%s\t%s\t%s\t%s\t%s\n' % (each_marker, metric_value_1, metric_score_1, metric_value_2, metric_score_2, metric_score_overall))
    summary_txt_handle.close()


if __name__ == '__main__':

    # initialize the options parser
    parser = argparse.ArgumentParser()
    parser.add_argument('-i', required=True, help='input file (e.g., nohup.out)')
    parser.add_argument('-o', required=True, help='output summary')
    args = vars(parser.parse_args())
    deltall(args)



from treeswift import *
from sys import stdin, stdout, argv, exit, stderr
import argparse
from os import path
import re
import io
import argparse
import arviz as az
import pandas as pd
import plotly.graph_objects as go
import logging
from sys import stdout
import numpy
import cvxopt
from numpy import *
import logging
from sys import stdout


FastRoot_usage = '''
================================= FastRoot example commands =================================

TreeSAK FastRoot -m1 r1_mcmc.txt -m2 r1_mcmc.txt -o convergence_plot.png

# This script was modified based on the script FastRoot.py from Uyen Mai: 
https://github.com/uym2/MinVar-Rooting/blob/master/FastRoot.py

==============================================================================================
'''


def new_logger(myName,myLevel=logging.INFO,myStream=stdout):
    logger = logging.getLogger(myName)
    logger.setLevel(myLevel)
    handler = logging.StreamHandler(myStream)


class Tree_extend(object):
    def __init__(self, ddpTree=None, tree_file=None, schema="newick"):  # ,logger_id=1,logger_stream=sys.stderr):
        # self.logger = new_logger(__name__+ "_" + str(logger_id),myStream=logger_stream)
        if tree_file:
            self.ddpTree = read_tree(tree_file, schema)
        else:
            self.ddpTree = ddpTree

    def Bottomup_label(self):
        # assign each node a label so that we can later relate to it
        i = 0
        for node in self.ddpTree.traverse_postorder():
            if node.is_leaf():
                node.name = 'L' + str(i)
            else:
                node.name = 'I' + str(i)
            i += 1

    def Topdown_label(self, label_type="all"):
        # assign each node a label so that we can later relate to it
        i = 0

        for node in self.ddpTree.traverse_preorder():
            if node.is_leaf():
                if label_type == "all" or label_type == "leaves":
                    node.name = 'L' + str(i)
                else:
                    node.name = node.label
            else:
                if label_type == "all" or label_type == "internal":
                    node.name = 'I' + str(i)
                else:
                    node.name = node.label
            i += 1

    def Bottomup_update(self):
        for node in self.ddpTree.traverse_postorder():
            self.Node_init(node)
            self.bUp_update(node)

    def Topdown_update(self):
        for node in self.ddpTree.traverse_preorder():
            self.tDown_update(node, self.Opt_function)

    def compute_distances(self):
        D = {}

        def __compute_dRoot__(node, cumm_l):
            if node.is_leaf():
                D[node.name] = cumm_l
            else:
                for child in node.child_nodes():
                    __compute_dRoot__(child, cumm_l + child.edge_length)

        __compute_dRoot__(self.ddpTree.root, 0)
        return D

    def compute_ingroup_distances(self):
        D = []

        def __compute_dLeaf__(node, cumm_l):
            if node.is_leaf():
                D.append(cumm_l)
            else:
                for child in node.child_nodes():
                    __compute_dLeaf__(child, cumm_l + child.edge_length)

        children = self.ddpTree.root.child_nodes()
        crowded_child = None
        maxleaf = -1

        for node in children:
            if node.nleaf > maxleaf:
                maxleaf = node.nleaf
                crowded_child = node

        __compute_dLeaf__(children[1], 0)

        return D

    def filter_branch(self, threshold=None):
        # filter out abnormally long branches
        i = 1
        self.logger.info("Iteration: " + str(i))
        self.Reroot()
        while 1:
            check = self.filter_by_threshold(threshold=threshold)
            if (not check):
                self.logger.info("I could not remove anything more! I stop here!")
                break
            i += 1
            self.logger.info("Iteration: " + str(i))
            self.reset()
            self.Reroot()

    def filter_by_threshold(self, threshold=None, k=3.5):
        if threshold is None:
            threshold = self.compute_threshold(k=k)

        def __filter__(node, cumm_l):
            removed = False
            node.child_removed = False
            for child in node.child_nodes():
                check = __filter__(child, cumm_l + child.edge_length)
                removed = removed or check

            p = node.parent_node
            # if ( cumm_l > threshold ) or ( node.child_removed and len(node.child_nodes()) == 0 ):
            if (cumm_l > threshold) or (node.child_removed and node.num_children() == 0):
                # remove node
                p.remove_child(node)
                # update parent node
                p.child_removed = True
                removed = True
                try:
                    self.logger.info(node.label + " removed")
                except:
                    self.logger.info(node.name + " removed")
            # elif len(node.child_nodes()) == 1:
            elif node.num_child_nodes() == 1:
                # remove node and attach its only child to its parent
                e1 = node.edge_length
                child = node.child_nodes()[0]
                e2 = child.edge_length
                p.remove_child(node)
                node.remove_child(child)
                p.add_child(child)
                child.edge_length = e1 + e2
            return removed

        return __filter__(self.get_root(), 0)

    def compute_threhold(self, k=3.5):
        self.logger.warning("Abstract class! Should never be called")
        return 0

    def reset(self):
        self.logger.warning("Abstract class! Should never be called")

    def find_root(self):
        self.Topdown_label()  # temporarily included for debugging
        self.Bottomup_update()
        self.prepare_root()
        self.Topdown_update()

    def opt_score(self):
        self.logger.warning("Abstract class! Should never be called")

    def report_score(self):
        self.logger.warning("Abstract class! Should never be called")

    def Reroot(self):
        self.find_root()
        # self.report_score()
        # d2currRoot = 0
        # br2currRoot = 0
        if self.opt_root != self.ddpTree.root:
            # d2currRoot,br2currRoot = self.reroot_at_edge(self.opt_root.edge, self.opt_root.edge_length-self.opt_x, self.opt_x)
            self.reroot_at_edge(self.opt_root, self.opt_x)
            # self.ddpTree.reroot(self.opt_root,self.opt_x)

        # return head_id, tail_id, edge_length, self.opt_x
        # return d2currRoot,br2currRoot

    def Opt_function(self, node):
        self.logger.warning("Abstract method! Should never be called")

    def tree_as_newick(self, outstream=sys.stdout, label_by_name=False):
        # dendropy's method to write newick seems to have problem ...
        self.__write_newick(self.ddpTree.root, outstream, label_by_name=label_by_name)
        outstream.write(";\n")

    #            outstream.write(bytes(";\n", "ascii"))

    def __write_newick(self, node, outstream, label_by_name=False):
        if node.is_leaf():
            if label_by_name:
                outstream.write(str(node.name))
            #                    outstream.write(bytes(str(node.name), "ascii"))
            else:
                try:
                    outstream.write(node.label)
                #                        outstream.write(bytes(node.label, "ascii"))
                except:
                    outstream.write(node.label)
        #                        outstream.write(bytes(str(node.label), "ascii"))
        else:
            outstream.write('(')
            # outstream.write(bytes('(', "ascii"))
            is_first_child = True
            for child in node.child_nodes():
                if is_first_child:
                    is_first_child = False
                else:
                    outstream.write(',')
                #                        outstream.write(bytes(',', "ascii"))
                self.__write_newick(child, outstream, label_by_name=label_by_name)
            outstream.write(')')
        #                outstream.write(bytes(')', "ascii"))
        if not node.is_leaf():
            if label_by_name:
                outstream.write(str(node.name))
            #                    outstream.write(bytes(str(node.name), "ascii"))
            elif node.label is not None:
                outstream.write(str(node.label))
        #                    outstream.write(bytes(str(node.label), "ascii"))

        if not node.edge_length is None:
            outstream.write(":" + str(node.edge_length))

    #                outstream.write(bytes(":" + str(node.edge_length), "ascii"))

    def reroot_at_edge(self, node, length):
        # the method provided by dendropy DOESN'T seem to work ...
        # change edge to opt_root
        length1 = node.edge_length - length
        length2 = length
        if not node:
            return
        head = node  # opt_root = v = node
        tail = node.parent  # u parent of opt_root
        if not tail:
            return

        if (length2 == 0) and head.is_leaf():
            return 0, 0

        # new_root = self.ddpTree.node_factory()
        new_root = Node()

        tail.remove_child(head)

        new_root.add_child(head)
        head.edge_length = length2

        p = tail.parent
        l = tail.edge_length

        new_root.add_child(tail)
        tail.edge_length = length1

        br2currRoot = 0
        d2currRoot = length1

        #            if tail.label == self.ddpTree.root.label:
        if (tail is self.ddpTree.root):
            head = new_root

        while tail is not self.ddpTree.root:
            # MAD@ add
            # q = tail.parent #tail should have 2 parents right now: new_root and its old parent
            q = head.parent
            # End MAD@ add
            head = tail
            tail = p
            p = tail.parent

            br2currRoot += 1
            d2currRoot += l

            l1 = tail.edge_length
            tail.remove_child(head)
            # MAD@ add
            head.parent = q
            # End MAD@ add

            head.add_child(tail)
            tail.edge_length = l
            l = l1

        # out of while loop: tail IS now tree.root
        if tail.num_children() == 1:
            # merge the 2 branches of the old root and adjust the branch length
            # sis = [child for child in tail.child_nodes()][0]
            sis = tail.child_nodes()[0]
            l = sis.edge_length
            tail.remove_child(sis)
            head.add_child(sis)
            sis.edge_length = l + tail.edge_length
            head.remove_child(tail)
            # tail.remove_child(head)

        new_root.name = self.ddpTree.root.name
        self.ddpTree.root.name = "OLD"
        self.ddpTree.root = new_root

        ### MAD@ add
        #            for node in self.ddpTree.traverse_postorder():
        #                for child in node.child_nodes():
        #                    if child.parent_node is not node:
        #                        logger.info("Error found!")
        #                        child.parent_node = node
        ### MAD@ add

        return d2currRoot, br2currRoot

    def get_root(self):
        return self.ddpTree.root


def cvxopt_solve_qp(P, q, G=None, h=None, A=None, b=None, maxIter=1000):
    P = .5 * (P + P.T)  # make sure P is symmetric
    args = [cvxopt.matrix(P), cvxopt.matrix(q)]
    if G is not None:
        args.extend([cvxopt.matrix(G), cvxopt.matrix(h)])
        if A is not None:
            args.extend([cvxopt.matrix(A), cvxopt.matrix(b)])
    sol = cvxopt.solvers.qp(*args,options={'show_progress':False,'maxiters':maxIter})
    if 'optimal' not in sol['status']:
        if "unknown" in sol['status']:
            logger.warning("Couldn't find optimal solution on one branch. Perhaps due to maximum iterations exceeded. Consider increasing the maximum iterations via -x.")
        else:
            logger.warning("Couldn't find optimal solution on one branch. Solution status: " + sol['status'])
    #return None
    return numpy.array(sol['x']).reshape((P.shape[1],))

EPSILON = 1e-5

class RTT_Tree(Tree_extend):
    # supportive base class to implement RTT-reroot, hence the name
    def __init__(self, smplTimes, ddpTree=None, tree_file=None, schema="newick", logger_id=1, logger_stream=stderr,
                 maxIter=1000):
        super(RTT_Tree, self).__init__(ddpTree, tree_file, schema)
        self.logger = new_logger("RTT_Tree_" + str(logger_id), myStream=logger_stream)
        self.smplTimes = smplTimes
        self.reset()
        self.maxIter = maxIter

    def reset(self):
        self.RTT = None
        self.opt_root = self.ddpTree.root
        self.opt_y = 0
        self.opt_x = 0
        self.opt_mu = 0
        self.tmin = min(self.smplTimes.values())

    def Node_init(self, node, nleaf=1, SDI=0, SD=0, ST=0, SDT=0, SSD=0):
        node.SDI = SDI
        node.SD = SD
        node.nleaf = nleaf
        node.ST = ST
        node.SDT = SDT
        node.SSD = SSD

    def Opt_function(self, node, SST, deltaT, deltaD, SDT, SSD, ST, SD):
        n = self.total_leaves
        a, b, c, d, e, f = n, SST, (-2 * deltaT), (2 * deltaD), (-2 * SDT), SSD
        k, m, r = 2 * (n - 2 * node.nleaf), -2 * ST, 2 * SD

        tmin = self.tmin

        # use quadprog to compute mu_star, y_star, and x_star
        P = array([[a, k / 2, c / 2.], [k / 2, n, m / 2], [c / 2, m / 2, b]])
        q = array([d / 2., r / 2, e / 2])
        G = array([[-1., 0., 0.], [0., 0., -1.], [1., 0., 0.], [0., 1., -tmin]])
        h = array([0., EPSILON, node.edge_length, 0]).reshape((4,))
        solution = cvxopt_solve_qp(P, q, G, h, maxIter=self.maxIter)
        x_star = solution[0]
        y_star = solution[1]
        mu_star = solution[2]
        curr_RTT = a * x_star * x_star + b * mu_star * mu_star + c * x_star * mu_star + d * x_star + e * mu_star + f + n * y_star * y_star + k * x_star * y_star + m * mu_star * y_star + r * y_star

        if self.RTT is None or (curr_RTT - self.RTT < -EPSILON):
            self.RTT = curr_RTT
            self.opt_root = node
            self.opt_x = node.edge_length - x_star
            self.opt_y = y_star
            self.opt_mu = mu_star

    def bUp_update(self, node):
        if node.is_leaf():
            node.nleaf = 1
            node.SDI = 0
            node.ST = self.smplTimes[node.label]
        else:
            node.nleaf = 0
            node.SDI = 0
            node.ST = 0
            for child in node.child_nodes():
                node.nleaf += child.nleaf
                node.SDI += child.SDI + child.nleaf * child.edge_length
                node.ST += child.ST

    def Update_var(self, child, node, edge_length):
        SST = self.SST
        deltaT = self.ddpTree.root.ST - 2 * child.ST
        deltaD = -2 * child.nleaf * edge_length - 2 * child.SDI + node.SD
        SDT = node.SDT
        SSD = node.SSD
        ST = self.ST
        SD = node.SD
        return SST, deltaT, deltaD, SDT, SSD, ST, SD

    def tDown_update(self, node, opt_function):
        for child in node.child_nodes():
            child.SD = node.SD + (self.total_leaves - 2 * child.nleaf) * child.edge_length
            child.SDT = node.SDT + child.edge_length * (self.ddpTree.root.ST - 2 * child.ST)
            child.SSD = node.SSD + (self.total_leaves - 4 * child.nleaf) * (child.edge_length ** 2) + 2 * (
                        node.SD - 2 * child.SDI) * child.edge_length
            SST, deltaT, deltaD, SDT, SSD, ST, SD = self.Update_var(child, node, child.edge_length)
            opt_function(child, SST, deltaT, deltaD, SDT, SSD, ST, SD)

    def prepare_root(self):
        root = self.get_root()
        root.SD = root.SDI
        self.total_leaves = root.nleaf
        self.ST = root.ST
        self.ddpTree.root.droot = 0
        self.ddpTree.root.troot = 0
        root.SD, root.SSD, root.SDT, self.SST = 0, 0, 0, 0
        for v in self.ddpTree.traverse_preorder():
            if not v.is_root():
                # must have defined edge lengths
                v.droot = v.parent.droot + v.edge_length
                if v.is_leaf():
                    root.SSD += (v.droot ** 2)
                    self.SST += (self.smplTimes[v.label] ** 2)
                    root.SD += v.droot
                    root.SDT += (v.droot * self.smplTimes[v.label])

    def opt_score(self):
        return self.RTT

    def return_values(self):
        # returns RTT score, mu, t0
        return (self.opt_score() / self.total_leaves), (self.opt_mu), (self.opt_y / self.opt_mu)

    def report_score(self):
        return "RTT=" + str(self.opt_score() / self.total_leaves) + "\tmu=" + str(self.opt_mu) + "\tt0=" + str(
            self.opt_y / self.opt_mu)


class OGR_Tree(Tree_extend):
    # supportive class to implement outgroup-reroot (OGR = outgroup reroot, hence the name)
    # this rooting method solve the difficulty in finding the root when there are mulitple outgroups
    # and they are not monophyletic. It seeks for the rooting place that maximizes the triplet score
    # of the specified outgroups.
    def __init__(self, outgroups, ddpTree=None, tree_file=None, schema="newick", logger_id=1, logger_stream=sys.stderr):
        super(OGR_Tree, self).__init__(ddpTree, tree_file, schema)
        self.logger = new_logger("OGR_Tree_" + str(logger_id), myStream=logger_stream)
        # L = self.ddpTree.leaf_nodes()
        L = []
        for leaf in self.ddpTree.traverse_leaves():
            L.append(leaf)
        self.OGs = set([x.label for x in L if x.label in set(outgroups)])
        self.nOGs = len(self.OGs)
        self.nIGs = len(L) - self.nOGs
        self.max_nTrpls = self.nIGs * self.nOGs * (self.nOGs - 1) / 2 + self.nOGs * self.nIGs * (self.nIGs - 1) / 2
        self.reset()

    def reset(self):
        self.opt_root = self.ddpTree.root
        self.opt_nTrpls = 0

    def Node_init(self, node, nTrpl_in=0, nTrpl_out=0, nOGs=0, nIGs=0):
        node.nTrpl_in = nTrpl_in
        node.nTrpl_out = nTrpl_out
        node.nOGs = nOGs
        node.nIGs = nIGs

    def Opt_function(self, node):
        curr_nTrpls = node.nTrpl_in + node.nTrpl_out
        if curr_nTrpls > self.opt_nTrpls:
            self.opt_nTrpls = curr_nTrpls
            self.opt_root = node
            self.opt_x = node.edge_length / 2  # NOTE: this method does not consider branch length, the *middle point* of the edge is just arbitrarily chosen

    def bUp_update(self, node):
        if node.is_leaf():
            node.nOGs = 1 if node.label in self.OGs else 0
            node.nIGs = 1 if node.nOGs == 0 else 0
        else:
            C = node.child_nodes()

            node.nOGs = sum([c.nOGs for c in C])
            node.nIGs = sum([c.nIGs for c in C])

            node.nTrpl_in = sum([c.nTrpl_in for c in C])

            for i, c1 in enumerate(C):
                for c2 in C[i + 1:]:
                    IG_trpls = c1.nIGs * c2.nIGs * (self.nOGs - node.nOGs)
                    OG_trpls = c1.nOGs * c2.nOGs * (self.nIGs - node.nIGs)
                    node.nTrpl_in += IG_trpls + OG_trpls

    def tDown_update(self, node, opt_function):
        C = node.child_nodes()

        for child in C:
            C1 = [c for c in C if c is not child]
            child.nTrpl_out = node.nTrpl_out

            for i, c1 in enumerate(C1):
                child.nTrpl_out += c1.nTrpl_in
                child.nTrpl_out += (self.nIGs - node.nIGs) * c1.nIGs * child.nOGs
                child.nTrpl_out += (self.nOGs - node.nOGs) * c1.nOGs * child.nIGs

                for c2 in C1[i + 1:]:
                    IG_trpls = c1.nIGs * c2.nIGs * child.nOGs
                    OG_trpls = c1.nOGs * c2.nOGs * child.nIGs

                    child.nTrpl_out += IG_trpls + OG_trpls

            opt_function(child)

    def prepare_root(self):
        pass

    def opt_score(self):
        return self.opt_nTrpls / float(self.max_nTrpls) if self.max_nTrpls != 0 else None

    def report_score(self):
        myScore = self.opt_score()
        if myScore is None:
            self.logger.warning("OG rooting failed because the tree has no outgroup")
        return "Triplet score: " + str(self.opt_score())


class MPR_Tree(Tree_extend):
    # supportive class to implement midpoint-reroot (mpr = mid point reroot, hence the name)G
    def __init__(self, ddpTree=None, tree_file=None, schema="newick", logger_id=1, logger_stream=sys.stderr):
        super(MPR_Tree, self).__init__(ddpTree, tree_file, schema)
        self.logger = new_logger("MPR_Tree_" + str(logger_id), myStream=logger_stream)
        self.reset()

    def reset(self):
        self.max_distance = -1
        self.opt_root = self.ddpTree.root
        self.opt_x = 0

    def Node_init(self, node, max_in=None, max_out=-1):
        node.max_in = max_in if max_in else [0, 0]
        node.max_out = max_out

    def Opt_function(self, node):
        m = max(node.max_in)
        curr_max_distance = m + node.max_out
        x = (node.max_out - m) / 2
        if curr_max_distance > self.max_distance and x >= 0 and x <= node.edge_length:
            self.max_distance = curr_max_distance
            self.opt_x = x
            self.opt_root = node

    def bUp_update(self, node):
        if not node.is_leaf():
            node.max_in = []
            for child in node.child_nodes():
                node.max_in.append(max(child.max_in) + child.edge_length)

    def tDown_update(self, node, opt_function):
        child_idx = 0
        for child in node.child_nodes():
            child.max_out = max([node.max_out] + [node.max_in[k] for k in range(len(node.max_in))
                                                  if k != child_idx]) + child.edge_length
            opt_function(child)
            child_idx += 1

    def prepare_root(self):
        pass

    def compute_threhold(self, k=3.5):
        self.logger.warning("Trying to compute threshold for MPR_Tree, which is not supported.")
        return 0

    def opt_score(self):
        return self.max_distance / 2

    def report_score(self):
        return "Tree height: " + str(self.opt_score())


class minVAR_Base_Tree(Tree_extend):
    # supportive base class to implement VAR-reroot, hence the name
    def __init__(self, ddpTree=None, tree_file=None, schema="newick",logger_id=1,logger_stream=sys.stderr):
        super(minVAR_Base_Tree, self).__init__(ddpTree, tree_file, schema)
        self.logger = new_logger("MinVar_Tree_" + str(logger_id),myStream=logger_stream)
        self.reset()

    def reset(self):
        self.minVAR = None
        self.opt_root = self.ddpTree.root
        self.opt_x = 0

    def Node_init(self, node, nleaf=1, sum_in=0, sum_total=0, var=-1):
        node.sum_in = sum_in
        node.sum_total = sum_total
        node.nleaf = nleaf
        node.var = var

    def Opt_function(self, node, a, b, c):
        self.logger.info("Abstract method! Should never be called")

    def compute_dRoot_VAR(self):
        cumm = {'ssq': 0, 'sum': 0}

        def compute_dRoot(node, cumm_l):
            if node.is_leaf():
                cumm['ssq'] += cumm_l ** 2
                cumm['sum'] += cumm_l
            else:
                for child in node.child_nodes():
                    compute_dRoot(child, cumm_l + child.edge_length)

        compute_dRoot(self.get_root(), 0)
        N = self.get_root().nleaf
        root_var = cumm['ssq'] / N - (cumm['sum'] / N) ** 2
        self.get_root().var = root_var

    def bUp_update(self, node):
        if node.is_leaf():
            node.nleaf = 1
            node.sum_in = 0
        else:
            node.nleaf = 0
            node.sum_in = 0
            for child in node.child_nodes():
                node.nleaf += child.nleaf
                node.sum_in += child.sum_in + child.nleaf * child.edge_length

    def Update_var(self, child, node, edge_length):
        alpha = 2 * (node.sum_total - 2 * (child.sum_in + child.nleaf * edge_length)) / self.total_leaves
        beta = 1 - 2 * float(child.nleaf) / self.total_leaves
        a = 1 - beta * beta
        b = alpha - 2 * node.sum_total * beta / self.total_leaves
        c = node.var
        child.var = a * edge_length * edge_length + b * edge_length + c
        return a, b, c

    def tDown_update(self, node, opt_function):
        for child in node.child_nodes():
            child.sum_total = node.sum_total + (self.total_leaves - 2 * child.nleaf) * child.edge_length
            a, b, c = self.Update_var(child, node, child.edge_length)
            opt_function(child, a, b, c)

    def prepare_root(self):
        root = self.get_root()
        root.sum_total = root.sum_in
        self.compute_dRoot_VAR()
        self.total_leaves = root.nleaf

    def opt_score(self):
        return self.minVAR

    def report_score(self):
        return "MinVar score: " + str(self.opt_score())


class MVDF_Tree(minVAR_Base_Tree):
    # supportive class to implement VAR-reroot + deepest node + factorization
    def __init__(self, ddpTree=None, tree_file=None, schema="newick"):
        super(MVDF_Tree, self).__init__(ddpTree, tree_file, schema)
        self.deep_node = None

    def reset(self):
        super(MVDF_Tree, self).reset()
        self.deep_node = None

    def Opt_function(self, node, a, b, c):
        x = -b / (2 * a)
        if x >= 0 and x <= node.edge_length:
            #                curr_minVAR = a*x*x + b*x + c
            factor = float(node.nleaf) / self.total_leaves
            factor = factor * (1 - factor)
            curr_minVAR = (a * x * x + b * x + c) / factor

            if node.var < node.parent_node.var:
                deep_node = node
            else:
                deep_node = node.parent_node

            updateNeed = False
            if (self.deep_node is None) or (deep_node.var < self.deep_node.var):
                self.deep_node = deep_node
                self.minVAR = curr_minVAR
                updateNeed = True
            elif (self.deep_node is deep_node) and (curr_minVAR < self.minVAR):
                self.minVAR = curr_minVAR
                updateNeed = True

            if updateNeed:
                self.opt_root = node
                self.opt_x = node.edge_length - x

    #                self.logger.info(str(curr_minVAR) + "\t" + node.label
    #                      + "\t" + str(node.edge_length-x) + "\t" + str(self.Tree_records[node.idx].var)
    #                      + "\t" + (str(node.parent_node.label) if node.parent_node else "None")
    #                      + "\t" + str(self.Tree_records[node.parent_node.idx].var))

    def compute_threshold(self, k=3.5):
        # should be called only AFTER the MV root was found
        mean = (self.opt_root.sum_total - self.opt_x *
                (self.total_leaves - 2 * self.opt_root.nleaf)) / self.total_leaves
        factor = float(self.opt_root.nleaf) / self.total_leaves
        factor = factor * (1 - factor)
        rootVar = self.minVAR * factor
        self.logger.info(mean)
        self.logger.info(rootVar)
        std = math.sqrt(rootVar)
        return mean + k * std


class MVD0_Tree(minVAR_Base_Tree):
    # supportive class to implement VAR-reroot + deepest node + no factorization
    def __init__(self, ddpTree=None, tree_file=None, schema="newick"):
        super(MVD0_Tree, self).__init__(ddpTree, tree_file, schema)
        self.deep_node = None

    def reset(self):
        super(MVD0_Tree, self).reset()
        self.deep_node = None

    def Opt_function(self, node, a, b, c):
        x = -b / (2 * a)
        if x >= 0 and x <= node.edge_length:
            curr_minVAR = a * x * x + b * x + c

            if node.var < node.parent_node.var:
                deep_node = node
            else:
                deep_node = node.parent_node

            updateNeed = False
            if (self.deep_node is None) or (deep_node.var < self.deep_node.var):
                self.deep_node = deep_node
                self.minVAR = curr_minVAR
                updateNeed = True
            elif (self.deep_node is deep_node) and (curr_minVAR < self.minVAR):
                self.minVAR = curr_minVAR
                updateNeed = True

            if updateNeed:
                self.opt_root = node
                self.opt_x = node.edge_length - x

    #                self.logger.info(str(curr_minVAR) + "\t" + node.label
    #                      + "\t" + str(node.edge_length-x) + "\t" + str(self.Tree_records[node.idx].var)
    #                      + "\t" + (str(node.parent_node.label) if node.parent_node else "None")
    #                      + "\t" + str(self.Tree_records[node.parent_node.idx].var))

    def compute_threshold(self, k=3.5):
        # should be called only AFTER the MV root was found
        mean = (self.opt_root.sum_total - self.opt_x *
                (self.total_leaves - 2 * self.opt_root.nleaf)) / self.total_leaves
        self.logger.info(mean)
        self.logger.info(self.minVAR)
        std = math.sqrt(self.minVAR)
        return mean + k * std


class MV0F_Tree(minVAR_Base_Tree):
    # supportive class to implement VAR-reroot + no deepest node + factorization
    #        def __init__(self, ddpTree = None, tree_file = None, schema = "newick"):
    #            super().__init__(ddpTree, tree_file, schema)

    def Opt_function(self, node, a, b, c):
        x = -b / (2 * a)
        if x >= 0 and x <= node.edge_length:
            #                curr_minVAR = a*x*x + b*x + c
            factor = float(node.nleaf) / self.total_leaves
            factor = factor * (1 - factor)
            curr_minVAR = (a * x * x + b * x + c) / factor
            if self.minVAR is None or curr_minVAR < self.minVAR:
                self.minVAR = curr_minVAR
                self.opt_root = node
                self.opt_x = node.edge_length - x

    #                self.logger.info(str(curr_minVAR) + "\t" + node.label
    #                      + "\t" + str(node.edge_length-x) + "\t" + str(self.Tree_records[node.idx].var)
    #                      + "\t" + (str(node.parent_node.label) if node.parent_node else "None")
    #                      + "\t" + str(self.Tree_records[node.parent_node.idx].var))

    def compute_threshold(self, k=3.5):
        # should be called only AFTER the MV root was found
        mean = (self.opt_root.sum_total - self.opt_x *
                (self.total_leaves - 2 * self.opt_root.nleaf)) / self.total_leaves
        factor = float(self.opt_root.nleaf) / self.total_leaves
        factor = factor * (1 - factor)
        rootVar = self.minVAR * factor
        self.logger.info(mean)
        self.logger.info(rootVar)
        std = math.sqrt(rootVar)
        return mean + k * std


class MV00_Tree(minVAR_Base_Tree):
    # supportive class to implement VAR-reroot + no deepest node + no factorization
    #        def __init__(self, ddpTree = None, tree_file = None, schema = "newick"):
    #            super().__init__(ddpTree, tree_file, schema)

    def Opt_function(self, node, a, b, c):
        x = -b / (2 * a)
        if x >= 0 and x <= node.edge_length:
            curr_minVAR = a * x * x + b * x + c
            if self.minVAR is None or curr_minVAR < self.minVAR:
                self.minVAR = curr_minVAR
                self.opt_root = node
                self.opt_x = node.edge_length - x

    def compute_threshold(self, k=3.5):
        # should be called only AFTER the MV root was found
        mean = (self.opt_root.sum_total - self.opt_x *
                (self.total_leaves - 2 * self.opt_root.nleaf)) / self.total_leaves
        self.logger.info(mean)
        self.logger.info(self.minVAR)
        std = math.sqrt(self.minVAR)
        return mean + k * std


class MBR_Tree(Tree_extend):
    # supportive class to implement midpoint balance root
    def __init__(self, ddpTree=None, tree_file=None, schema="newick"):
        super(MBR_Tree, self).__init__(ddpTree, tree_file, schema)

        self.BPs = []  # BPs : balance points
        self.opt_root = self.ddpTree.root
        self.opt_x = 0

    def Node_init(self, node, nleaf=1, sum_in=0, sum_out=-1):
        self.nleaf = nleaf
        self.sum_in = sum_in
        self.sum_out = sum_out

    def Opt_function(self, node):
        nleaf = node.nleaf
        mean_in = node.sum_in / nleaf
        mean_out = node.sum_out / (self.total_leaves - nleaf)
        x = (mean_out - mean_in) / 2
        if x >= 0 and x <= node.edge_length:
            self.BPs.append((node, x, mean_in + x))
            node.x = x
            node.mean = mean_in + x
        else:
            node.x = None
            node.mean = None

    def bUp_update(self, node):
        node.sum_in = 0
        if node.is_leaf():
            node.nleaf = 1
        else:
            node.nleaf = 0
            for child in node.child_nodes():
                node.nleaf += child.nleaf
                node.sum_in += child.sum_in + child.nleaf * child.edge_length

    def tDown_update(self, node, opt_function):
        child_idx = 0
        for child in node.child_nodes():
            child.sum_out = (node.sum_out + node.sum_in + child.edge_length *
                             (self.total_leaves - 2 * child.nleaf) - child.sum_in)
            opt_function(child)
            child_idx += 1

    def prepare_root(self):
        root = self.get_root()
        root.sum_out = 0
        self.total_leaves = root.nleaf
        root.x = None
        root.mean = None

    def list_balance_points(self):
        self.Topdown_label()
        self.Bottomup_update()
        self.prepare_root()
        self.Topdown_update()

        for (node, x, mean) in self.BPs:
            if node.is_leaf():
                #                  self.logger.info(node.label + "\t" + str(x) + "\t" + str(mean))
                self.logger.info(node.label + "\t" + str(x) + "\t" + str(mean))
            else:
                self.logger.info(node.label + "\t" + str(x) + "\t" + str(mean))

    def build_balance_tree(self):
        self.Topdown_label()  # keep this step for now for debugging purpose
        self.Bottomup_update()
        self.prepare_root()
        self.Topdown_update()

        # self.list_balance_points()

        self.balance_tree = self.ddpTree.extract_tree()

        # bottom up pruning
        for node in self.balance_tree.traverse_postorder():
            node.type = "real"
            node.BPbelow = False

            '''if node.is_leaf():
                self.logger.info("parent: " + node.label)# + "\t" + str(node.extraction_source.x))
            else:
                self.logger.info("parent: " + node.label)#+ "\t" + str(node.extraction_source.x))'''

            for ch in node.child_nodes():
                '''try:
                    self.logger.info("child: " + ch.label)# + "\t" + str(ch.extraction_source.x))
                except:
                    self.logger.info("child: " + ch.label) #+ "\t" + str(ch.extraction_source.x))'''

                if ch.BPbelow or (ch.extraction_source.x is not None):
                    node.BPbelow = True
                # node.BPbelow = node.BPbelow or ch.BPbelow or (ch.extraction_source.x is not None)

                if not ch.BPbelow:
                    # remove the whole clade under ch
                    # for ch1 in ch.child_nodes():
                    #    ch.remove_child(ch1)
                    edgelen = ch.edge_length
                    node.remove_child(ch)

                    if ch.extraction_source.x is not None:
                        # add a new node p at the balance point
                        # set p to be a child of node (edge length ch.edge_length - x)
                        # add a new node ch1 to be another child of p (edge length ch.mean)
                        edgelen = ch.edge_length

                        # p = self.ddpTree.node_factory()
                        # ch1 = self.ddpTree.node_factory()
                        p = Node()
                        ch1 = Node()

                        p.type = "bp"  # bp: balance-point
                        p.ref_child = ch.extraction_source  # link p to the original tree (for later use after finding midpoint)
                        ch1.type = "dm"  # dm: dummy

                        # node.remove_child(ch)
                        node.add_child(p)
                        p.add_child(ch1)

                        p.edge_length = edgelen - ch.extraction_source.x
                        ch1.edge_length = ch.extraction_source.mean

                elif ch.extraction_source.x is not None:
                    # add a new node p at the balance point
                    # set p to be a child of node (edge length ch.edge_length - x)
                    # set ch to be a child of p (edge length x)
                    # add a new node ch1 to be another child of p (edge length ch.mean)

                    edgelen = ch.edge_length

                    # p = self.ddpTree.node_factory()
                    p = Node()
                    # ch1 = self.ddpTree.node_factory()
                    ch1 = Node()

                    p.type = "bp"
                    p.ref_child = ch.extraction_source  # link p to the original tree (for later use after finding midpoint)
                    ch1.type = "dm"

                    node.remove_child(ch)
                    node.add_child(p)
                    p.add_child(ch)
                    p.add_child(ch1)

                    ch.edge_length = ch.extraction_source.x
                    p.edge_length = edgelen - ch.extraction_source.x
                    ch1.edge_length = ch.extraction_source.mean

                    # topdown pruning
        node = self.balance_tree.root
        nchild = len(node.child_nodes())
        while nchild > 0 and nchild < 2:
            # node has less than 2 children
            temp = node
            node = node.child_nodes()[0]
            temp.remove_child(node)
            if node.type == "dm":
                node = temp
                break
            nchild = len(node.child_nodes())

        self.balance_tree.root = node
        self.balance_tree.root.edge_length = None
        # balance_tree.root = None

        # mptre = MPR_Tree(ddpTree=balance_tree)
        # mptre.tree_as_newick()

        # return balance_tree

    def find_root(self):
        self.build_balance_tree()
        mptre = MPR_Tree(ddpTree=self.balance_tree)
        mptre.tree_as_newick()
        mptre.find_root()

        self.logger.info(mptre.opt_root.type)

        if mptre.opt_root.type == "bp":
            self.opt_root = mptre.opt_root.ref_child
            self.opt_x = mptre.opt_root.ref_child.x + mptre.opt_x
        elif mptre.opt_root.type == "dm":
            self.logger.info("Hmm... Is it possible that a dummy was found as the opt_root?")
        else:
            self.opt_root = mptre.opt_root.extraction_source
            self.opt_x = mptre.opt_x

        self.logger.info(self.opt_root.label)
        self.logger.info(self.opt_x)


def FastRoot(args):

    stream = args.infofile if args.infofile else stderr
    logger = new_logger(__name__, myStream=stream)
    logger.info("Running " + fastroot.PROGRAM_NAME + " version " + fastroot.PROGRAM_VERSION)
    new_argv = re.sub(' +', ' ', " ".join(argv).replace("\n", " "))
    logger.info(fastroot.PROGRAM_NAME + " was called as follows: " + new_argv)

    METHOD2FUNC = {'MP': MPR_Tree, 'MV': MV00_Tree, 'OG': OGR_Tree, 'RTT': RTT_Tree}
    METHOD2DESC = {'MP': "Midpoint", 'MV': "MinVar", 'OG': "Outgroup", 'RTT': "Root-to-Tip"}
    method = args.method.upper()

    # reading outgroups
    if args.outgroups:
        if method != 'OG':
            method = 'OG'
            logger.warning("The rooting method is set to outgroup rooting (OG) due to the presence of outgroups")
        if path.exists(args.outgroups):
            OGs = []
            for line in open(args.outgroups, 'r'):
                OGs.append(line.strip())
        else:
            OGs = args.outgroups.split()
    else:
        OGs = None

    # reading sampling times
    if args.smplTimes:
        smplTimes = {}
        for line in args.smplTimes:
            sp, t = line.strip().split()
            smplTimes[sp] = float(t)
        if method != 'RTT':
            method = 'RTT'
            logger.warning(
                "The rooting method is set to root-to-tip rooting (RTT) due to the presence of sampling times")

    if method == 'RTT' and args.smplTimes is None:
        logger.error("Need sampling times for root-to-tip rooting")
        exit()
    elif method == 'OG' and args.outgroups is None:
        logger.error("Need outgroups for outgroup rooting")
        exit()

    assert method in METHOD2FUNC, "Invalid method! Valid options: MP for midpoint, MV for minVAR, OG for outgroups, RTT for root-to-tip"
    logger.info("Rooting Method: " + METHOD2DESC[method] + " Rooting")

    if method == 'RTT':
        if args.maxIter and args.maxIter < 1000:
            logger.warning(
                "Invalid number of maximum iterations (-x). Must be at least 1000. Set back to 1000 by default.")
        maxIter = max(1000, args.maxIter) if args.maxIter else 1000
        logger.info("Maximum iterations: " + str(maxIter))
    elif args.maxIter is not None:
        logger.warning("The maximum number of iterations (-x) is only used with root-to-tip rooting (RTT)")

    # read and root each tree
    for i, line in enumerate(args.input):
        tree = read_tree(line, schema=args.schema.lower())
        if method == 'OG':
            a_tree = OGR_Tree(OGs, ddpTree=tree, logger_id=i + 1, logger_stream=stream)
        elif method == 'RTT':
            a_tree = RTT_Tree(smplTimes, ddpTree=tree, logger_id=i + 1, logger_stream=stream, maxIter=maxIter)
        else:
            a_tree = METHOD2FUNC[method](ddpTree=tree, logger_id=i + 1, logger_stream=stream)

        a_tree.Reroot()
        logger.info("Tree " + str(i + 1) + " " + a_tree.report_score())
        a_tree.tree_as_newick(outstream=args.outfile)


if __name__ == '__main__':

    parser = argparse.ArgumentParser()
    parser.add_argument('-i', '--input',        required=False, type=argparse.FileType('r'), default=stdin, help="Input File (default is STDIN)")
    parser.add_argument('-m', '--method',       required=False, type=str, default="MV", help="Method (MP for midpoint, MV for minVAR, OG for outgroup, RTT for root-to-tip) (default is MV)")
    parser.add_argument('-g', '--outgroups',    required=False, type=str, help="Specify the outgroups. If specifying a list of outgroups, put them between quotes (i.e. \"). Otherwise, specifying a file which containts all the outgroups. Can only be used with -m OG")
    parser.add_argument('-t', '--smplTimes',    required=False, type=argparse.FileType('r'), help="The file containing the sampling times at leaves; to be used with -m RTT")
    parser.add_argument('-o', '--outfile',      required=False, type=argparse.FileType('w'), default=stdout, help="Output File (default is STDOUT)")
    parser.add_argument('-s', '--schema',       required=False, type=str, default="newick", help="Schema of your input treefile (default is newick)")
    parser.add_argument('-f', '--infofile',     required=False, type=argparse.FileType('w'), default=None, help="Save all the logging to this file. Default: print to stderr")
    parser.add_argument("-x", "--maxIter",      required=False, type=int, default=None, help="Maximum number of iterations to run cvxopt")
    args = vars(parser.parse_args())
    FastRoot(args)


import os
import argparse
import arviz as az
import pandas as pd
import matplotlib as mpl
mpl.use('Agg')
import matplotlib.pyplot as plt
from matplotlib.pyplot import figure


CompareMCMC_usage = '''
====================================== CompareMCMC example commands ======================================

TreeSAK CompareMCMC -mx IR_mcmc.txt -my AR_mcmc.txt -lx IR -ly AR -o convergence_plot.png -max 40 -fs 12

cd /Users/songweizhi/Desktop
TreeSAK CompareMCMC -mx /Users/songweizhi/Desktop/Sponge_r220/6_dating/MCMCTree/dating_outputs/topo2p10_clock3_nsample250000_run1_mcmc.txt -my /Users/songweizhi/Desktop/Sponge_r220/6_dating/MCMCTree/dating_outputs/topo2p10_clock3_nsample250000_run2_mcmc.txt -lx IR -ly AR -o convergence_plot.png -max 40 -fs 12

==========================================================================================================
'''


def sep_path_basename_ext(file_in):
    file_path, file_name = os.path.split(file_in)
    if file_path == '':
        file_path = '.'
    file_basename, file_extension = os.path.splitext(file_name)
    return file_path, file_basename, file_extension


def CompareMCMC(args):

    mcmc_txt_x      = args['mx']
    mcmc_txt_y      = args['my']
    label_x         = args['lx']
    label_y         = args['ly']
    pwd_figure      = args['o']
    max_axis_value  = args['max']
    label_fs        = args['fs']

    x_path, x_basename, x_ext = sep_path_basename_ext(mcmc_txt_x)
    y_path, y_basename, y_ext = sep_path_basename_ext(mcmc_txt_y)

    if label_x is None:
        label_x = x_basename
    if label_y is None:
        label_y = y_basename

    # read in dataframe
    df_x = pd.read_table(mcmc_txt_x, index_col=0)
    df_y = pd.read_table(mcmc_txt_y, index_col=0)

    # get Mean value for each column
    df_x_col_to_mean_dict = {col_name: mean for col_name, mean in df_x.mean().iteritems()}
    df_y_col_to_mean_dict = {col_name: mean for col_name, mean in df_y.mean().iteritems()}

    # get CI95 for each column
    df_x_col_to_ci_dict = {col_name: az.hdi(col.values, hdi_prob=0.95) for col_name, col in df_x.iteritems()}
    df_y_col_to_ci_dict = {col_name: az.hdi(col.values, hdi_prob=0.95) for col_name, col in df_y.iteritems()}

    num_list_x = []
    num_list_y = []
    err_range_x = []
    err_range_y = []
    for col_name, col in df_x.iteritems():
        if col_name not in ['mu', 'sigma2', 'lnL']:
            num_list_x.append(df_x_col_to_mean_dict[col_name])
            num_list_y.append(df_y_col_to_mean_dict[col_name])
            err_range_x.append(df_x_col_to_ci_dict[col_name])
            err_range_y.append(df_y_col_to_ci_dict[col_name])

    x_err_l = []
    x_err_r = []
    y_err_l = []
    y_err_u = []
    max_value = 0
    min_value = 100000000000000
    n = 0
    while n < len(num_list_x):
        x_value = num_list_x[n]
        y_value = num_list_y[n]
        x_range = err_range_x[n]
        y_range = err_range_y[n]
        x_l_dist = abs(x_value - x_range[0])
        x_r_dist = abs(x_range[1] - x_value)
        y_l_dist = abs(y_value - y_range[0])
        y_u_dist = abs(y_range[1] - y_value)
        x_err_l.append(x_l_dist)
        x_err_r.append(x_r_dist)
        y_err_l.append(y_l_dist)
        y_err_u.append(y_u_dist)

        current_max = max(x_value, y_value, x_range[0], x_range[1], y_range[0], y_range[1])
        current_min = min(x_value, y_value, x_range[0], x_range[1], y_range[0], y_range[1])

        if current_max > max_value:
            max_value = current_max
        if current_min < min_value:
            min_value = current_min
        n += 1

    figure(figsize=(6, 6), dpi=300)
    plt.plot([min_value, max_value], [min_value, max_value], color='black', linestyle='dashed', linewidth=1, alpha=0.5)
    plt.scatter(num_list_x, num_list_y, s=0)
    plt.errorbar(num_list_x, num_list_y, xerr=[x_err_l, x_err_r], yerr=[y_err_l, y_err_u],
                 ls='none', ecolor='skyblue', elinewidth=1, alpha=0.5)

    if max_axis_value is not None:
        plt.xlim([0, max_axis_value])
        plt.ylim([0, max_axis_value])

    # Set the font size of xticks and yticks
    plt.xticks(fontsize=label_fs)
    plt.yticks(fontsize=label_fs)
    plt.xlabel(label_x, fontsize=label_fs)
    plt.ylabel(label_y, fontsize=label_fs)

    # write out
    plt.tight_layout()
    plt.savefig(pwd_figure)
    plt.close()

    print('Plot exported to %s, done!' % pwd_figure)


if __name__ == '__main__':

    # initialize the options parser
    parser = argparse.ArgumentParser()
    parser.add_argument('-mx',      required=True,                          help='mcmc.txt for x axis')
    parser.add_argument('-my',      required=True,                          help='mcmc.txt for y axis')
    parser.add_argument('-lx',      required=False, default=None,           help='label for x axis')
    parser.add_argument('-ly',      required=False, default=None,           help='label for y axis')
    parser.add_argument('-max',     required=False, default=None, type=int, help='maximum axis value')
    parser.add_argument('-fs',      required=False, default=16, type=int,   help='label font size, default: 16')
    parser.add_argument('-o',       required=True,                          help='output plot')
    args = vars(parser.parse_args())
    CompareMCMC(args)



import os
import glob
import argparse
from ete3 import Tree
from Bio import AlignIO
import multiprocessing as mp


supertree_usage = '''
====================== supertree example commands ======================

Dependencies: mafft, trimal, bmge and iqtree2

TreeSAK supertree -i best10 -x fa -o best10_astral_tree -bmge -t 12 -f

========================================================================
'''


def sep_path_basename_ext(file_in):

    f_path, f_name = os.path.split(file_in)
    if f_path == '':
        f_path = '.'
    f_base, f_ext = os.path.splitext(f_name)

    return f_name, f_path, f_base, f_ext[1:]


def fa2phy(fasta_in, phy_out):

    alignment = AlignIO.read(fasta_in, 'fasta')

    max_seq_id_len = 0
    for each_seq in alignment:
        seq_id_len = len(each_seq.id)
        if seq_id_len > max_seq_id_len:
            max_seq_id_len = seq_id_len

    with open(phy_out, 'w') as msa_out_handle:
        msa_out_handle.write('%s %s\n' % (len(alignment), alignment.get_alignment_length()))
        for each_seq in alignment:
            seq_id = each_seq.id
            seq_id_with_space = '%s%s' % (seq_id, ' ' * (max_seq_id_len + 2 - len(seq_id)))
            msa_out_handle.write('%s%s\n' % (seq_id_with_space, str(each_seq.seq)))


def PB(msa_in, op_dir, op_prefix, fa_to_plp, num_of_threads, num_of_chains, force_overwrite):

    ####################################################################################################################

    msa_in_name, msa_in_path, msa_in_base, msa_in_ext = sep_path_basename_ext(msa_in)

    settings_dombrowski = '-cat -gtr -x 10 -1 -dgam 4'
    setting_to_use  = settings_dombrowski
    msa_in_plp      = '%s/%s.phylip'        % (op_dir, msa_in_base)
    cmd_txt         = '%s/%s_cmds.txt'      % (op_dir, msa_in_base)

    ####################################################################################################################

    # create output dir
    if os.path.isdir(op_dir) is True:
        if force_overwrite is True:
            os.system('rm -r %s' % op_dir)
        else:
            print('output folder already exist, program exited!')
            exit()
    os.system('mkdir %s' % op_dir)

    # fa_to_phylip
    msa_to_use = msa_in
    if fa_to_plp is True:
        fa2phy(msa_in, msa_in_plp)
        msa_to_use = msa_in_plp

    cores_per_chain = 0
    chain_name_list = []
    pb_mpi_cmd_list = []
    jobs_to_run_in_parallel = 0
    if num_of_chains == 1:
        jobs_to_run_in_parallel = 1
        pb_mpi_cmd = 'mpirun -np %s pb_mpi -d %s %s -s %s/%s' % (num_of_threads, msa_to_use, setting_to_use, op_dir, op_prefix)
        chain_name_list.append('%s/%s' % (op_dir, op_prefix))
        pb_mpi_cmd_list.append(pb_mpi_cmd)
        cores_per_chain = num_of_threads

    elif num_of_threads <= num_of_chains:
        jobs_to_run_in_parallel = num_of_threads
        for chain_index in range(1, (num_of_chains + 1)):
            current_wd = '%s/%s_chain%s' % (op_dir, op_prefix, chain_index)
            os.mkdir(current_wd)
            pb_mpi_cmd = 'mpirun -np %s pb_mpi -d %s %s -s %s/%s_chain%s' % (1, msa_to_use, setting_to_use, current_wd, op_prefix, chain_index)
            chain_name_list.append('%s/%s_chain%s' % (current_wd, op_prefix, chain_index))
            pb_mpi_cmd_list.append(pb_mpi_cmd)
            cores_per_chain = 1
    else:
        jobs_to_run_in_parallel = num_of_chains
        cores_per_run = num_of_threads // num_of_chains
        for chain_index in range(1, (num_of_chains + 1)):
            current_wd = '%s/%s_chain%s' % (op_dir, op_prefix, chain_index)
            os.mkdir(current_wd)
            pb_mpi_cmd = 'mpirun -np %s pb_mpi -d %s %s -s %s/%s_chain%s' % (cores_per_run, msa_to_use, setting_to_use, current_wd, op_prefix, chain_index)
            chain_name_list.append('%s/%s_chain%s' % (current_wd, op_prefix, chain_index))
            pb_mpi_cmd_list.append(pb_mpi_cmd)
            cores_per_chain = cores_per_run

    # write out commands
    cmd_txt_handle = open(cmd_txt, 'w')
    for cmd in pb_mpi_cmd_list:
        cmd_txt_handle.write(cmd + '\n')

    cmd_txt_handle.write('\n# To restart a terminated run (e.g., due to walltime limitation)\n')
    for each_chain in chain_name_list:
        cmd_txt_handle.write('mpirun -np %s pb_mpi %s\n' % (cores_per_chain, each_chain))
    cmd_txt_handle.close()

    # run chains with mp
    print('Running pb_mpi with multiprocessing')
    pool = mp.Pool(processes=jobs_to_run_in_parallel)
    pool.map(os.system, pb_mpi_cmd_list)
    pool.close()
    pool.join()

    # assess the results
    if num_of_chains > 1:

        readpb_cmd = 'bpcomp -x 1000 10 %s' % (' '.join(chain_name_list))
        bpcomp_cmd = 'tracecomp -x 1000 %s' % (' '.join(chain_name_list))

        # write out commands
        cmd_txt_handle = open(cmd_txt, 'a')
        cmd_txt_handle.write(readpb_cmd + '\n')
        cmd_txt_handle.write(bpcomp_cmd + '\n')
        cmd_txt_handle.close()

        # report
        print('You may want to use the following commands to assess the results:')
        print(readpb_cmd)
        print(bpcomp_cmd)

    print('Done!')


def supertree(args):

    oma_op_fasta            = args['i']
    fasta_file_ext          = args['x']
    op_dir                  = args['o']
    trim_with_bmge          = args['bmge']
    trim_model              = args['bmge_m']
    entropy_score_cutoff    = args['bmge_esc']
    iqtree_model            = args['iqtree_m']
    force_overwrite         = args['f']
    num_of_threads          = args['t']
    infer_pb_tree           = args['pb']

    # specify path to BMGE.jar
    current_file_path = '/'.join(os.path.realpath(__file__).split('/')[:-1])
    pwd_bmge_jar = '%s/BMGE.jar' % current_file_path

    fa_file_re   = '%s/*.%s' % (oma_op_fasta, fasta_file_ext)
    fa_file_list = ['.'.join(os.path.basename(i).split('.')[:-1]) for i in glob.glob(fa_file_re)]

    if len(fa_file_list) == 0:
        print('No file found in %s, program exited!' % oma_op_fasta)
        exit()

    ################################################################################

    # define file name
    cmd_1_mafft_txt         = '%s/cmd_1_mafft.txt'          % op_dir
    cmd_2_trim_txt          = '%s/cmd_2_trim.txt'           % op_dir
    cmd_3_tree_txt          = '%s/cmd_3_tree.txt'           % op_dir
    cmd_4_astral_txt        = '%s/cmd_4_astral.txt'         % op_dir
    aln_dir                 = '%s/dir_1_msa'                % op_dir
    trimmed_aln_dir         = '%s/dir_2_trimmed_msa'        % op_dir
    tree_dir                = '%s/dir_3_tree'               % op_dir
    combined_gene_tree_file = '%s/combined_trees.txt'       % op_dir
    astral_mapping_txt      = '%s/name_mapping.txt'         % op_dir
    consensus_tree_txt      = '%s/consensus_tree.treefile'  % op_dir

    ################################################################################

    # create output folder
    if os.path.isdir(op_dir) is True:
        if force_overwrite is True:
            os.system('rm -r %s' % op_dir)
        else:
            print('%s exist, program exited!' % op_dir)
            exit()

    os.mkdir(op_dir)
    os.mkdir(aln_dir)
    os.mkdir(trimmed_aln_dir)
    os.mkdir(tree_dir)

    ################################################################################

    cmd_list_mafft = []
    cmd_list_trim  = []
    cmd_list_tree  = []
    cmd_1_mafft_txt_handle = open(cmd_1_mafft_txt, 'w')
    cmd_2_trim_txt_handle = open(cmd_2_trim_txt, 'w')
    cmd_3_tree_txt_handle = open(cmd_3_tree_txt, 'w')
    for each_og in sorted(fa_file_list):

        # define file name
        current_gene_tree_dir   = '%s/dir_3_tree/%s'    % (op_dir, each_og)
        og_fa                   = '%s/%s.%s'            % (oma_op_fasta, each_og, fasta_file_ext)
        og_aln                  = '%s/%s.aln'           % (aln_dir, each_og)
        og_aln_trimmed          = '%s/%s_trimal.aln'    % (trimmed_aln_dir, each_og)
        if trim_with_bmge is True:
            og_aln_trimmed      = '%s/%s_bmge.aln'      % (trimmed_aln_dir, each_og)

        os.system('mkdir %s' % current_gene_tree_dir)

        # prepare commands
        mafft_cmd          = 'mafft-einsi --thread %s --quiet %s > %s'                                             % (1, og_fa, og_aln)

        trim_cmd           = 'trimal -in %s -out %s -automated1'                                                   % (og_aln, og_aln_trimmed)
        if trim_with_bmge is True:
            trim_cmd       = 'java -jar %s -i %s -m %s -t AA -h %s -of %s'                                         % (pwd_bmge_jar, og_aln, trim_model, entropy_score_cutoff, og_aln_trimmed)

        infer_tree_cmd     = 'iqtree2 -s %s --seqtype AA -m %s -B 1000 --wbtl --bnni --prefix %s/%s -T %s --quiet' % (og_aln_trimmed, iqtree_model, current_gene_tree_dir, each_og, num_of_threads)
        if infer_pb_tree is True:
            infer_tree_cmd = 'TreeSAK PB -i %s -o %s -p %s -t %s -n %s -fa2plp'                                    % (og_aln_trimmed, current_gene_tree_dir, each_og, num_of_threads, 4)

        # add commands to list
        cmd_list_mafft.append(mafft_cmd)
        cmd_list_trim.append(trim_cmd)
        cmd_list_tree.append(infer_tree_cmd)

        # write out commands
        cmd_1_mafft_txt_handle.write(mafft_cmd + '\n')
        cmd_2_trim_txt_handle.write(trim_cmd + '\n')
        cmd_3_tree_txt_handle.write(infer_tree_cmd + '\n')

    cmd_1_mafft_txt_handle.close()
    cmd_2_trim_txt_handle.close()
    cmd_3_tree_txt_handle.close()

    # run mafft commands
    print('Running mafft with %s cores for %s OGs' % (num_of_threads, len(fa_file_list)))
    pool = mp.Pool(processes=num_of_threads)
    pool.map(os.system, cmd_list_mafft)
    pool.close()
    pool.join()

    # run trim commands
    print('Trimming with %s cores for %s OGs' % (num_of_threads, len(fa_file_list)))
    pool = mp.Pool(processes=num_of_threads)
    pool.map(os.system, cmd_list_trim)
    pool.close()
    pool.join()

    # run iqtree commands
    if infer_pb_tree is False:
        print('Running iqtree with %s cores for %s OGs' % (num_of_threads, len(fa_file_list)))
        for each_iqtree_cmd in sorted(cmd_list_tree):
            print(each_iqtree_cmd)
            os.system(each_iqtree_cmd)
    else:
        print('Commands for inferring PhyloBayes tree exported to %s' % cmd_3_tree_txt)

    #################################################### run astral ####################################################

    if infer_pb_tree is False:

        # cat gene trees
        os.system('cat %s/*.treefile > %s' % (tree_dir, combined_gene_tree_file))

        gnm_to_gene_dict = dict()
        for each_tree in open(combined_gene_tree_file):
            tree_str = each_tree.strip()
            current_tree = Tree(tree_str, quoted_node_names=True, format=1)
            for node in current_tree.traverse():
                if node.is_leaf():
                    leaf_name = node.name
                    leaf_gnm = '_'.join(leaf_name.split('_')[:-1])
                    if leaf_gnm not in gnm_to_gene_dict:
                        gnm_to_gene_dict[leaf_gnm] = {leaf_name}
                    else:
                        gnm_to_gene_dict[leaf_gnm].add(leaf_name)

        # get the mapping file
        astral_mapping_txt_handle = open(astral_mapping_txt, 'w')
        for each_gnm in gnm_to_gene_dict:
            current_gene_set = gnm_to_gene_dict[each_gnm]
            for each_gene in current_gene_set:
                astral_mapping_txt_handle.write('%s\t%s\n' % (each_gene, each_gnm))
        astral_mapping_txt_handle.close()

        # may need to add more parameters
        astral_cmd = 'astral -i %s -o %s -t %s -a %s' % (combined_gene_tree_file, consensus_tree_txt, num_of_threads, astral_mapping_txt)
        # -r    --round     Integer 4       Number of initial rounds of placements
        # -s    --subsample Integer 4       Number of rounds of subsampling per exploration step

        # write out command
        cmd_4_astral_txt_handle = open(cmd_4_astral_txt, 'w')
        cmd_4_astral_txt_handle.write(astral_cmd + '\n')
        cmd_4_astral_txt_handle.close()

        # run astral
        os.system(astral_cmd)

    else:
        print('Things to do:')
        print('Run PhyloBayes with commands exported to %s' % cmd_3_tree_txt)
        print('Wait until your chains, for each of your protein family, reached convergence')
        print('1. get consensus gene tree for each protein family usingn')
        print('2. get species tree based on the consensus gene trees with trimal')

    ####################################################################################################################


if __name__ == '__main__':

    supertree_parser = argparse.ArgumentParser()
    supertree_parser.add_argument('-i',         required=True,                          help='orthologous gene sequence')
    supertree_parser.add_argument('-x',         required=True,                          help='faa file extension')
    supertree_parser.add_argument('-o',         required=True,                          help='output directory')
    supertree_parser.add_argument('-bmge',      required=False, action="store_true",    help='trim with BMGE, default is trimal')
    supertree_parser.add_argument('-bmge_m',    required=False, default='BLOSUM30',     help='trim model, default: BLOSUM30')
    supertree_parser.add_argument('-bmge_esc',  required=False, default='0.55',         help='entropy score cutoff, default: 0.55')
    supertree_parser.add_argument('-iqtree_m',  required=False, default='LG+G+I',       help='iqtree_model, default: LG+G+I')
    supertree_parser.add_argument('-pb',        required=False, action="store_true",    help='infer tree with PhyloBayes-MPI, default is iqtree')
    supertree_parser.add_argument('-f',         required=False, action="store_true",    help='force overwrite')
    supertree_parser.add_argument('-t',         required=False, type=int, default=1,    help='num of threads, default: 1')
    args = vars(supertree_parser.parse_args())
    supertree(args)
